/// \file 
/// \brief Definitions related to the AC class

#ifndef AC_H
#define AC_H

#include "constraint_selection.h"
#include "assignment.h"

class AC  {            /// This class allows to represent AC algorithms
	protected:
		Constraint_Selection * selection;			///< choice of a policy of constraint selection
		Multiple_Residue_Support * supports;	///< the structure of the supports
		bool onepass;													///< true if the arc-consistency is achieved in one-pass, false if the arc-consistency is achieved until the fix-point is reached		

	public:
		AC (Multiple_Residue_Support * ls, Constraint_Selection * cs, bool one_pass = false); 		///< constructor
		virtual ~AC ();			///< destructor
		
		// basic functions
		Multiple_Residue_Support * Get_Support ();                  ///< returns a pointer on the structure representing the supports
    bool Achieve (CSP * pb);																		///< achieves the current filtering algorithm on the CSP pb and returns true if the problem is consistent w.r.t. this algorithm, false otherwise 
		virtual bool Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds);	///< achieves the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
		virtual bool Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds, int var)=0;	///< achieves, from the variable var (if any), the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
};


//-------------------------------
// definition of inline function
//-------------------------------


// achieves the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
inline bool AC::Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds) {
	return Achieve (pb,A,ds,-1);
}


// returns a pointer on the structure representing the supports
inline Multiple_Residue_Support * AC::Get_Support (){
	return supports;
}

#endif

