#include "all_different_global_constraint.h"

//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a all-diff global constraint 
All_Different_Global_Constraint::All_Different_Global_Constraint (vector<Variable *> & var): Global_Constraint (var) {
  val = new int [arity];
  real_val = new int [arity];
  modified = new bool [arity];
  candidates = new int [arity];
}
		

// destructor
All_Different_Global_Constraint::~All_Different_Global_Constraint () {
  delete [] val;
  delete [] real_val;
  delete [] modified;
  delete [] candidates;
}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool All_Different_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  throw ("Error");
}


// returns true if the application of the propagator of the constraint w.r.t. the assignment A deletes a value from the domain of at least one variable from the scope, false otherwise (is_modified[i] is true if the i th variable of the scope has been modified, false otherwise)
bool All_Different_Global_Constraint::Propagate (CSP * pb, Assignment & A, Multiple_Residue_Support * ls, Deletion_Stack * ds, bool * & is_modified) {
	all_values.clear();
	bool modif = false;
	unsigned int nb = 0;
	unsigned int nb_done = 0;
	for (unsigned int i = 0; i < arity; i++) {
		modified[i] = false;
		if (A.Is_Assigned (scope_var[i]->Get_Num())) {
			all_values.insert(scope_var[i]->Get_Domain()->Get_Remaining_Real_Value(0));
			nb_done++;
		}
		else {
				candidates[nb] = i;
				nb++;
			}
	}
	is_modified = modified;
	
	if (nb < arity)	{
		// we remove all the values in all_values from the remaining domains
		for (unsigned int j = 0; j < nb; j++)	{
			Domain * d = scope_var[candidates[j]]->Get_Domain();		
			for (set<int>::iterator iter = all_values.begin(); (iter != all_values.end()) && (d->Get_Size() > 0); iter++)	{
				int v = d->Get_Value(*iter);

				if ((v != -1) && (d->Is_Present(v)))	{
					ds->Add_Element (d,v);
					d->Filter_Value (v);
					modified[candidates[j]] = true;
					
					modif = true;
				}
			}
			
			if (d->Get_Size () == 0)	{
				weight++;
				return true;
			}
		}
	}	
	
	// we consider the unassigned variables
	
	for (unsigned int i = 0; i < nb; i++)	{
		nb_done++;
		unsigned int min = scope_var[candidates[i]]->Get_Domain()->Get_Size();
		int ind_min = i;
		for (unsigned int j = i+1; j < nb; j++)
			if (scope_var[candidates[j]]->Get_Domain()->Get_Size() < min)	{
				min = scope_var[candidates[j]]->Get_Domain()->Get_Size();
				ind_min = j;
			}
		
		// we update de the set all_values
		Domain * d = scope_var[candidates[ind_min]]->Get_Domain();		
		for (unsigned int j = 0; j < d->Get_Size(); j++)	all_values.insert(d->Get_Remaining_Real_Value(j));
	
		// we remove the selected candidate
		candidates[ind_min] = candidates[i];

		if (all_values.size() == nb_done)
			// we remove all the values in all_values from the remaining domains
			for (unsigned int j = i+1; j < nb; j++) {
				d = scope_var[candidates[j]]->Get_Domain();		
				for (set<int>::iterator iter = all_values.begin(); (iter != all_values.end()) && (d->Get_Size() > 0); iter++)	{
					int v = d->Get_Value(*iter);

					if ((v != -1) && (d->Is_Present(v)))	{
						ds->Add_Element (d,v);
						d->Filter_Value (v);
						modified[candidates[j]] = true;
						
						modif = true;						
					}
				}
				
				if (d->Get_Size () == 0)	{
					weight++;
					return true;
				}
			}
	}
	
	return modif;
}


// returns true if the application of the propagator of the constraint deletes a value from the domain of at least one variable from the scope, false otherwise (is_modified[i] is true if the i th variable of the scope has been modified, false otherwise)
bool All_Different_Global_Constraint::Propagate (CSP * pb, Multiple_Residue_Support * ls, Deletion_Stack * ds, bool * & is_modified) {
	bool modif = false;
	for (unsigned int i = 0; i < arity; i++) {
		modified[i] = false;
		candidates[i] = i;
	}
	is_modified = modified;
	all_values.clear();
	
	for (unsigned int i = 0; i < arity; i++) {
		unsigned int min = scope_var[candidates[i]]->Get_Domain()->Get_Size();
		int ind_min = i;
		for (unsigned int j = i+1; j < arity; j++)
			if (scope_var[candidates[j]]->Get_Domain()->Get_Size() < min) {
				min = scope_var[candidates[j]]->Get_Domain()->Get_Size();
				ind_min = j;
			}
		
		// we update de the set all_values
		Domain * d = scope_var[candidates[ind_min]]->Get_Domain();		
		for (unsigned int j = 0; j < d->Get_Size(); j++)
			all_values.insert(d->Get_Remaining_Real_Value(j));
	

		// we remove the selected candidate
		candidates[ind_min] = candidates[i];

	
		if (all_values.size() == i+1) {
			// we remove all the values in all_values from the remaining domains
			for (unsigned int j = i+1; j < arity; j++) {
				d = scope_var[candidates[j]]->Get_Domain();		
				for (set<int>::iterator iter = all_values.begin(); (iter != all_values.end()) && (d->Get_Size() > 0); iter++) {
					int v = d->Get_Value(*iter);

					if ((v != -1) && (d->Is_Present(v))) {
						ds->Add_Element (d,v);
						d->Filter_Value (v);
						modified[candidates[j]] = true;
						
						modif = true;						
					}
				}
				
				if (d->Get_Size () == 0) {
					weight++;
					return true;
				}
			}
		}
	}
	
	return modif;
}
