/// \file 
/// \brief Definitions related to the All_Different_Global_Constraint class

#ifndef ALL_DIFFERENT_GLOBAL_CONSTRAINT_H
#define ALL_DIFFERENT_GLOBAL_CONSTRAINT_H


#include "global_constraint.h"

class All_Different_Global_Constraint: public Global_Constraint  {    /// This class implements the all-diff global constraint
  protected:
    int * val;     		   		///< the values representing a tuple
    int * real_val;  		 		///< the values representing a real tuple
    bool * modified;				///< the variables whose domain has been modified by the last call to the method Propagate (modified[i] is true if at least one value has been removed from the domain of the ith variable in the scope, false otherwise)
    set<int> all_values;		///< the set of values which belong currently to the domain of at least one variable in the constraint's scope
    int * candidates;				///< the next domains to proceed in the method Propagate
  
	public:
		// constructors and destructor
		All_Different_Global_Constraint (vector<Variable *> & var);              ///< constructs a new all-diff global constraint
		~All_Different_Global_Constraint ();														         ///< destructor
				
		// basic functions
		virtual bool Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds);	 ///< returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
		virtual bool Has_Specific_Propagator ();		///< returns true if the constraint has a specific propagator, false otherwise
		virtual bool Propagate (CSP * pb, Multiple_Residue_Support * ls, Deletion_Stack * ds, bool * & is_modified);	 ///< returns true if the application of the propagator of the constraint deletes a value from the domain of at least one variable from the scope, false otherwise (is_modified[i] is true if the i th variable of the scope has been modified, false otherwise)
		virtual bool Propagate (CSP * pb, Assignment & A, Multiple_Residue_Support * ls, Deletion_Stack * ds, bool * & is_modified);	 ///< returns true if the application of the propagator of the constraint w.r.t. the assignment A deletes a value from the domain of at least one variable from the scope, false otherwise (is_modified[i] is true if the i th variable of the scope has been modified, false otherwise)
};


//-----------------------------
// inline function definitions
//-----------------------------


// returns true if the constraint has a specific propagator, false otherwise
inline bool All_Different_Global_Constraint::Has_Specific_Propagator (){
	return true;
}

#endif

