/// \file 
/// \brief Definitions related to the Assignment class

#ifndef ASSIGNMENT_H
#define ASSIGNMENT_H

#include "csp.h"
#include <light_vector.h>
#include <light_stack.h>

class Assignment  {        /// This class allows to represent assignments
	protected:
		Light_Stack<pair<int,int> > values;   ///< the assignment of value to variable 
		Light_Vector<int> position;           ///< the position of each variable in the assginment
		
	public:
		// constructors and destructor
		Assignment (int max_size);			  ///< construct an empty assignment whose maximum size is max_size
		Assignment (Assignment & a);		  ///< construct an assignment by copying the assignment a
		
		// operator
		Assignment & operator= (Assignment & a);   ///< operator =
			
		// basic functions
		unsigned int Get_Size ();							///< returns the number of assigned variables in the current assignment
		void Assign (int var, int val);				///< assignes the value val to the variable var in the current assignment
		void Reassign (int var, int val);			///< changes the assignment of the variable var to the value val in the current assignment
		void Unassign ();											///< unassignes the last assigned variable in the current assignment
		void Unassign (int var);							///< unassignes the variable var in the current assignment
		void Clear ();												///< unassignes all the variables
		int Last_Assigned ();									///< returns the last assigned variable (-1 if the assignment is empty)
		bool Is_Assigned (int var);						///< returns true if the variable var is assigned, false otherwise
		int Get_Variable (unsigned int pos);	///< returns the number of the variable assigned at position pos, -1 if there are less than pos assigned variables
		int Get_Value (int var);							///< returns the value of the assigned variable var
		int Get_Position (int var);						///< returns the position of the variable var, -1 if the variable is not assigned
};


//-----------------------------
// inline function definitions
//-----------------------------


// returns the number of assigned variables in the current assignment
inline unsigned int Assignment::Get_Size () {
	return values.size();
}


// assignes the value val to the variable var in the current assignment
inline void Assignment::Assign (int var, int val) {
	position[var] = values.size();
	values.push (pair<int,int>(var,val));	
}


// changes the assignment of the variable var to the value val in the current assignment
inline void Assignment::Reassign (int var, int val) {
	values[position[var]].second = val;	
}


// unassignes the last assigned variable in the current assignment
inline void Assignment::Unassign () {
	position[values.top().first] = -1;
	values.pop();	
}


// returns the last assigned variable (-1 if the assignment is empty)
inline int Assignment::Last_Assigned () {
	if (values.size () == 0)  return -1;
	else return values.top().first;
}


// returns true if the variable var is assigned, false otherwise
inline bool Assignment::Is_Assigned (int var) {
	return position[var] != -1;
}


// returns the number of the variable assigned at position pos, -1 if there are less than pos assigned variables
inline int Assignment::Get_Variable (unsigned int pos) {
	if ((pos < 0) || (pos >= values.size()))  return -1;
	else return values[pos].first;
}


// returns the value of the assigned variable var
inline int Assignment::Get_Value (int var) {
	return values[position[var]].second;
}


// returns the position of the variable var, -1 if the variable is not assigned
inline int Assignment::Get_Position (int var) {
	return position[var];
}
#endif
