#include "auxiliary_functions_main.h"

// constraint types
#include "predicate_constraint.h"
#include "less_than_sum_arithmetic_global_constraint.h"
#include "less_equal_sum_arithmetic_global_constraint.h"
#include "greater_than_sum_arithmetic_global_constraint.h"
#include "greater_equal_sum_arithmetic_global_constraint.h"
#include "equal_sum_arithmetic_global_constraint.h"
#include "not_equal_sum_arithmetic_global_constraint.h"
#include "not_equal_weighted_sum_arithmetic_global_constraint.h"
#include "less_than_weighted_sum_arithmetic_global_constraint.h"
#include "less_equal_weighted_sum_arithmetic_global_constraint.h"
#include "greater_than_weighted_sum_arithmetic_global_constraint.h"
#include "greater_equal_weighted_sum_arithmetic_global_constraint.h"
#include "equal_weighted_sum_arithmetic_global_constraint.h"


// returns true if e is a sum, false otherwise
bool is_sum (Expression & e) {
	if ((e.Get_Type() == OP) && (e.Get_Operator() == ADD))
		if (e.Get_Subexpression(0).Get_Type() == VAR)
			if (e.Get_Subexpression(1).Get_Type() == VAR) return true;
			else return is_sum (e.Get_Subexpression(1));
		else if (e.Get_Subexpression(1).Get_Type() == VAR)	return is_sum (e.Get_Subexpression(0));
         else return is_sum (e.Get_Subexpression(0)) && is_sum (e.Get_Subexpression(1));
	else return false;
}
	

// returns true if e is a sum compared to a constant, false otherwise
bool is_sum_comparison (Expression & e) {
	if ((e.Get_Type() == OP) && ((e.Get_Operator() == EQ) || (e.Get_Operator() == NE) || (e.Get_Operator() == LT) || (e.Get_Operator() == LE) || 
      (e.Get_Operator() == GT) || (e.Get_Operator() == GE)))
		if (e.Get_Subexpression(0).Get_Type() == VAL) 	return is_sum (e.Get_Subexpression(1));
		else if (e.Get_Subexpression(1).Get_Type() == VAL) 	return is_sum (e.Get_Subexpression(0));
         else return 0;
	else return false;
}

// returns true if e is a sum, false otherwise
bool is_weighted_sum (Expression & e) {
	if ((e.Get_Type() == OP) && ((e.Get_Operator() == ADD) || (e.Get_Operator() == SUM)))	{
		int arity = e.Get_Arity();
		bool ok = true;
		int i = 0;
		while ((i < arity) && (ok))	{
			if (e.Get_Subexpression(i).Get_Type() == VAR)  ok = true;
			else if (e.Get_Subexpression(i).Get_Type() == OP)
             if (e.Get_Subexpression(i).Get_Operator() == MUL)	{
               Expression p = e.Get_Subexpression(i);
               ok = ((p.Get_Subexpression(0).Get_Type() == VAR) && (p.Get_Subexpression(1).Get_Type() == VAL)) || ((p.Get_Subexpression(0).Get_Type() == VAL)
                       && (p.Get_Subexpression(1).Get_Type() == VAR));
             }
             else if (e.Get_Subexpression(i).Get_Operator() == NEG) 	ok = e.Get_Subexpression(i).Get_Subexpression(0).Get_Type() == VAR;
                  else ok = is_weighted_sum (e.Get_Subexpression(i));
           else ok = false;
			i++;
		}
		return ok;
	}
	else return false;
}


// returns true if e is a sum compared to a constant, false otherwise
bool is_weighted_sum_comparison (Expression & e) {
	if ((e.Get_Type() == OP) && ((e.Get_Operator() == EQ) || (e.Get_Operator() == NE) || (e.Get_Operator() == LT) || (e.Get_Operator() == LE) || 
      (e.Get_Operator() == GT) || (e.Get_Operator() == GE)))
		if (e.Get_Subexpression(0).Get_Type() == VAL) 	return is_weighted_sum (e.Get_Subexpression(1));
		else if (e.Get_Subexpression(1).Get_Type() == VAL)	return is_weighted_sum (e.Get_Subexpression(0));
			   else return 0;
	else return false;
}


// returns true if e is a sum s.t. X0+cs t compared to a variable, false otherwise
bool is_particular_weighted_sum_comparison (Expression & e) {
	if ((e.Get_Type() == OP) && ((e.Get_Operator() == EQ) || (e.Get_Operator() == NE) || (e.Get_Operator() == LT) || (e.Get_Operator() == LE) || 
      (e.Get_Operator() == GT) || (e.Get_Operator() == GE))) {
		if ((e.Get_Subexpression(0).Get_Type() == OP) && (e.Get_Subexpression(0).Get_Operator() == ADD) && (e.Get_Subexpression(1).Get_Type() == VAR) &&
				(((e.Get_Subexpression(0).Get_Subexpression(0).Get_Type() == VAR) && (e.Get_Subexpression(0).Get_Subexpression(1).Get_Type() == VAL)) ||
				 ((e.Get_Subexpression(0).Get_Subexpression(0).Get_Type() == VAL) && (e.Get_Subexpression(0).Get_Subexpression(1).Get_Type() == VAR))))
			return true;

		if ((e.Get_Subexpression(1).Get_Type() == OP) && (e.Get_Subexpression(1).Get_Operator() == ADD) && (e.Get_Subexpression(0).Get_Type() == VAR) &&
				(((e.Get_Subexpression(1).Get_Subexpression(0).Get_Type() == VAR) && (e.Get_Subexpression(1).Get_Subexpression(1).Get_Type() == VAL)) ||
				 ((e.Get_Subexpression(1).Get_Subexpression(0).Get_Type() == VAL) && (e.Get_Subexpression(1).Get_Subexpression(1).Get_Type() == VAR))))
			return true;
	}
	return false;
}


// returns the weight of each variable via the vector weights. We assume that the expression e is a weighted sum as defined in function is_weighted_sum
void Get_Sum_Weights (Expression & e, vector<int> & weights) {
	if ((e.Get_Type() == OP) && ((e.Get_Operator() == ADD) || (e.Get_Operator() == SUM))) {
		int arity = e.Get_Arity();
		for (int i = 0; i < arity; i++)
			if (e.Get_Subexpression(i).Get_Type() == VAR)
				weights[e.Get_Subexpression(i).Get_Variable()]++;
			else if (e.Get_Subexpression(i).Get_Type() == OP) {
             if (e.Get_Subexpression(i).Get_Operator() == MUL) {
               Expression p = e.Get_Subexpression(i);
               if (p.Get_Subexpression(0).Get_Type() == VAR) 	weights[p.Get_Subexpression(0).Get_Variable()] += p.Get_Subexpression(1).Get_Value();
               else weights[p.Get_Subexpression(1).Get_Variable()] += p.Get_Subexpression(0).Get_Value();
             }
             else if (e.Get_Subexpression(i).Get_Operator() == NEG)  weights[e.Get_Subexpression(i).Get_Subexpression(0).Get_Variable()]--;
                  else Get_Sum_Weights (e.Get_Subexpression(i),weights);
           }
	}
}


// replaces the particular sum by a weighted_sum
Expression replace_particular_weighted_sum (Expression & e) {
	Expression subexpression [2];
	Expression add[2];
	Expression mult[2];
	mult[0] = Expression (VAL,-1);
	
	if ((e.Get_Subexpression(0).Get_Type() == OP) && (e.Get_Subexpression(0).Get_Operator() == ADD) && (e.Get_Subexpression(1).Get_Type() == VAR)) {
		mult[1] = Expression (VAR,e.Get_Subexpression(1).Get_Variable());
		if ((e.Get_Subexpression(0).Get_Subexpression(0).Get_Type() == VAR) && (e.Get_Subexpression(0).Get_Subexpression(1).Get_Type() == VAL)) {
			add[0] = Expression (VAR,e.Get_Subexpression(0).Get_Subexpression(0).Get_Variable());
			subexpression[1] = Expression (VAL,-e.Get_Subexpression(0).Get_Subexpression(1).Get_Value());
		}
		else {
				add[0] = Expression (VAR,e.Get_Subexpression(0).Get_Subexpression(1).Get_Variable());
				subexpression[1] = Expression (VAL,-e.Get_Subexpression(0).Get_Subexpression(0).Get_Value());
			}			
	}
	else {
			add[0] = Expression (VAR,e.Get_Subexpression(0).Get_Variable());
			
			if ((e.Get_Subexpression(1).Get_Subexpression(0).Get_Type() == VAR) && (e.Get_Subexpression(1).Get_Subexpression(1).Get_Type() == VAL)) {
				mult[1] = Expression (VAR,e.Get_Subexpression(1).Get_Subexpression(0).Get_Variable());	
				subexpression[1] = Expression (VAL,e.Get_Subexpression(1).Get_Subexpression(1).Get_Value());
			}
			else {
					mult[1] = Expression (VAR,e.Get_Subexpression(1).Get_Subexpression(1).Get_Variable());	
					subexpression[1] = Expression (VAL,e.Get_Subexpression(1).Get_Subexpression(0).Get_Value());
				}				
		}

	add[1] = Expression (MUL,mult,2);
	subexpression[0] = Expression (ADD,add);
	
	Expression e2 = Expression (e.Get_Operator(),subexpression,2);
	
	return e2;
}


// create a new Weighted_Sum_Constraint constraint
Constraint * Create_Weighted_Sum_Constraint (Expression & e, vector<Variable *> scope_var) {
	int arity = scope_var.size();
	Constraint * c;
	vector<int> weights (arity,0);
	int constant;

	if (e.Get_Subexpression(0).Get_Type() == VAL) {
		constant = e.Get_Subexpression(0).Get_Value();
		Get_Sum_Weights (e.Get_Subexpression(1),weights);
	}
	else {
			constant = e.Get_Subexpression(1).Get_Value();
			Get_Sum_Weights (e.Get_Subexpression(0),weights);
		}
	
	switch (e.Get_Operator())	{
		case EQ: c = new Equal_Weighted_Sum_Arithmetic_Global_Constraint (scope_var,weights,constant); break;
		case NE: c = new Not_Equal_Weighted_Sum_Arithmetic_Global_Constraint (scope_var,weights,constant); break;
		case LT: c = new Less_Than_Weighted_Sum_Arithmetic_Global_Constraint (scope_var,weights,constant); break;
		case LE: c = new Less_Equal_Weighted_Sum_Arithmetic_Global_Constraint (scope_var,weights,constant); break;
		case GT: c = new Greater_Than_Weighted_Sum_Arithmetic_Global_Constraint (scope_var,weights,constant); break;
		case GE: c = new Greater_Equal_Weighted_Sum_Arithmetic_Global_Constraint (scope_var,weights,constant); break;
		default: c = 0;  // requires for removing some warnings at compilation step
	}
	
	return c;
}


// replaces some predicate constraints by global constraints
void Replace_Predicate_By_Global_Constraint (CSP * pb) {	
	Expression e2;
	
	for (unsigned int i = 0; i < pb->Get_M(); i++) {
		Constraint * c = pb->Get_Constraint (i);

		if (dynamic_cast<Predicate_Constraint *>(c) != 0)	{
			int arity = c->Get_Arity();
			Constraint * new_c = 0;
			Expression e;
			
			vector<Variable *> scope_var (arity);
			for (int j = 0; j < arity; j++)  scope_var[j] = c->Get_Scope_Variable(j);

			e = dynamic_cast<Predicate_Constraint *>(c)->Get_Relation();

      if (is_sum_comparison(e)) {
        int constant;
        if (e.Get_Subexpression(0).Get_Type() == VAL)  constant = e.Get_Subexpression(0).Get_Value();
        else constant = e.Get_Subexpression(1).Get_Value();

        switch (e.Get_Operator()) {
          case EQ: new_c = new Equal_Sum_Arithmetic_Global_Constraint (scope_var,constant); break;
          case NE: new_c = new Not_Equal_Sum_Arithmetic_Global_Constraint (scope_var,constant); break;
          case LT: new_c = new Less_Than_Sum_Arithmetic_Global_Constraint (scope_var,constant); break;
          case LE: new_c = new Less_Equal_Sum_Arithmetic_Global_Constraint (scope_var,constant); break;
          case GT: new_c = new Greater_Than_Sum_Arithmetic_Global_Constraint (scope_var,constant); break;
          case GE: new_c = new Greater_Equal_Sum_Arithmetic_Global_Constraint (scope_var,constant); break;
          default: new_c = 0;  // requires for removing some warnings at compilation step
        }
      }
      else if (is_weighted_sum_comparison(e))  new_c = Create_Weighted_Sum_Constraint (e,scope_var);
           else if (is_particular_weighted_sum_comparison (e)) {
                  Expression e2 = replace_particular_weighted_sum (e);
                  new_c = Create_Weighted_Sum_Constraint (e2,scope_var);
                }
								
			if (new_c != 0)  pb->Replace_Constraint (i,new_c);
		}
	}
}
