#include "avl_node.h"


//-----------------------------
// constructors and destructor 
//-----------------------------


// constructs a node with no child and a value val
AVL_Node::AVL_Node (int * val, int arity, AVL_Node *** & tuple_list) {
	value = new int [arity];
	next = new AVL_Node * [arity];
	prev = new AVL_Node * [arity];
	for (int i = 0; i < arity; i++)	{
		value[i] = val[i];
		next[i] = tuple_list[i][val[i]];
		prev[i] = 0;
		if (tuple_list[i][val[i]] != 0)  tuple_list[i][val[i]]->prev[i] = this;
		tuple_list[i][val[i]] = this;
	}
	
	imbalance = 0;
	lchild = 0;
	rchild = 0;	
}

		
// destructor
AVL_Node::~AVL_Node () {
	delete [] value;
	delete [] next;
	delete [] prev;
	delete lchild;
	delete rchild;
}


//-----------------
// basic functions 
//-----------------
	

// return 0 if e1 == e2, 1 if e1 < e2, -1 otherwise
int Compare_Values (int * e1, int * e2, int n) {
  int i;

  i = 0;

  while ((i < n) && (e1[i] == e2[i]))    i++;

  if (i == n)  return 0;
  else if (e1[i] < e2[i])  return 1;
       else return -1;
}


// returns true if the element val belongs to the AVL, false otherwise
bool AVL_Node::Is_Present (int * val, int arity) {
	AVL_Node * p;
  int comp;
  
  p = this;

  while ((p != 0) && (comp = Compare_Values (val,p->value,arity)))
    if (comp == 1)  p = p->lchild; 
    else p = p->rchild;

  if (p == 0) return false;
  else return true; 
}


// returns a pointer on the node containing the element val if it belongs to the AVL, 0 otherwise
AVL_Node * AVL_Node::Get_Node (int * val, int arity) {
	AVL_Node * p;
  int comp;
  
  p = this;

  while ((p != 0) && (comp = Compare_Values (val,p->value,arity)))
    if (comp == 1)  p = p->lchild; 
    else p = p->rchild;

  return p; 
}


// adds the element val into the AVL (if it does not belong to the AVL yet)
bool AVL_Node::Add_Element (AVL_Node * & A, int * val, int arity, AVL_Node *** & tuple_list) {
	bool H;  // true if the imbalance has increased, false otherwise
	
	if (A == 0) {
    // we insert the new value into a leaf */
		A = new AVL_Node (val,arity,tuple_list);
		
		return true;
	} 
	else {   
		switch (Compare_Values (val,A->value,arity)) {
			case 1 :   // e < root->value
					H = Add_Element (A->lchild,val,arity,tuple_list);
					if (H) {  
            // the height has increased 
					
						if (A->imbalance == -1) {
							if (Compare_Values (val,A->lchild->value,arity) == 1) { 
								// case left-left
								AVL_Node * B;

								B = A->lchild;
								A->lchild = B->rchild;
								B->rchild = A;  
								A->imbalance = 0;
								A = B; 
								A->imbalance = 0;
							}
							else	{ 
									// cas left-right
									AVL_Node *A1, *A2;
							
									A1 = A->lchild;
									A2 = A1->rchild;
									A1->rchild = A2->lchild;
									A2->lchild = A1;
									A->lchild = A2->rchild;
									A2->rchild = A;
									if (A2->imbalance == -1)  A->imbalance = 1;
									else A->imbalance = 0;

									if (A2->imbalance == 1)  A1->imbalance = -1;
									else A1->imbalance = 0; 

									A = A2;
									A->imbalance = 0;
								}     
							return false;
						}
						else if (A->imbalance == 1) { 
                   A->imbalance = 0; 
                   return false;
                 }
                 else if (A->imbalance == 0) {
                        A->imbalance = -1; 
                        return true;
                      }
					} 
					else return false; 
					break;  
		case 0 : return false;
		case -1: 
				H = Add_Element (A->rchild,val,arity,tuple_list);
				if (H) {
          // the heigth has increased
					if (A->imbalance == 1) {
						if (Compare_Values (A->rchild->value,val,arity) == 1) { 
							// case right-right
							AVL_Node * B;

							B = A->rchild;
							A->rchild = B->lchild;
							B->lchild = A;  
							A->imbalance = 0;
							A = B; 
							A->imbalance = 0;
						}
						else { 
								// case right-left
								AVL_Node *A1, *A2;
						
								A1 = A->rchild;
								A2 = A1->lchild;
								A1->lchild = A2->rchild;
								A2->rchild = A1;
								A->rchild = A2->lchild;
								A2->lchild = A;
								if (A2->imbalance == 1)  A->imbalance = -1;
								else A->imbalance = 0;

								if (A2->imbalance == -1)  A1->imbalance = 1;
								else A1->imbalance = 0; 
	
								A = A2;
								A->imbalance = 0;
							}
						return false;
					}  
					else if (A->imbalance == -1) { 
                 A->imbalance = 0; 
                 return false;
               }
               else if (A->imbalance == 0) {
                      A->imbalance = 1; 
                      return true;
                    }
				}
				else return false;
				break; 
		}
	}
	return false;
}
