/// \file 
/// \brief Definitions related to the AVL_Node class

#ifndef AVL_NODE_H
#define AVL_NODE_H


class AVL_Node {         /// This class allows to represent the nodes of a AVL tree
	private:
		int * value;				///< the value of the current node of the AVL
		int imbalance; 			///< the imbalance of the current node of the AVL (an integer between -1 and 1)
		AVL_Node * lchild;	///< the left child of the current node of the AVL
		AVL_Node * rchild;	///< the right child of the current node of the AVL
		AVL_Node ** next;		///< the next tuple for each value of the current node
		AVL_Node ** prev;		///< the previous tuple for each value of the current node
	
	public:
		// constructor and destructor
		AVL_Node (int * val, int arity, AVL_Node *** & tuple_list);			///< constructs a node with no child and a value val
		~AVL_Node ();																										///< destructor
		
		// basic functions
		bool Add_Element (AVL_Node * & A, int * val, int arity, AVL_Node *** & tuple_list);			///< adds the element val into the tree pointed by A (if it does not belong yet)
		bool Is_Present (int * val, int arity);											///< returns true if the element val belongs to the AVL, false otherwise
		AVL_Node * Get_Node (int * val, int arity);									///< returns a pointer on the node containing the element val if it belongs to the AVL, 0 otherwise
		int * Get_Tuple ();																					///< returns the current tuple
		AVL_Node * Next_Tuple (int pos);														///< returns the next tuple w.r.t. the variable at the position pos in the tuple	
};


// returns the current tuple
inline int * AVL_Node::Get_Tuple () {
	return value;
}


// returns the next tuple w.r.t. the variable at the position pos in the tuple
inline AVL_Node * AVL_Node::Next_Tuple (int pos) {
	return next[pos];
}

#endif
