/// \file 
/// \brief Definitions related to the Basic_Deletion_Stack class

#ifndef BASIC_DELETION_STACK_H
#define BASIC_DELETION_STACK_H

#include "csp.h"
#include "deletion_stack.h"

class Basic_Deletion_Stack: public Deletion_Stack  {     /// This class implements basic stacks of deletions
	protected:
		pair<Domain *,int> * elements;	///< stack represented in a vector of the elements (the elements are pair (variable,value))
		int * number_elements;				  ///< the number of elements in the stack for each level
		int size;                       ///< the current number of values in the stack
		int top; 											  ///< the top of the stack (i.e. the current level)

	public:
		// constructor and destructor
		Basic_Deletion_Stack (CSP * pb, int level_number);	 ///< construct an empty deletion stack related to the CSP pb with at most level_number levels
		~Basic_Deletion_Stack();														 ///< destructor
		
		
		// basic functions
		virtual void New_Level ();													///< adds a new level of stack (i.e. increases the top of the stack)
		virtual void Add_Element (Domain * dom, int val);		///< adds an element (dom,val) to the current level of the stack
		virtual void Restore (CSP * pb);										///< restores the elements at the top level and decreases the top of the stack
};


//-----------------------------
// inline function definitions
//-----------------------------


// adds a new level of stack (i.e. increases the top of the stack)
inline void Basic_Deletion_Stack::New_Level () {
	top++;
	number_elements[top] = 0;
}


// adds an element (dom,val) to the current level of the stack
inline void Basic_Deletion_Stack::Add_Element (Domain * dom, int val) {
	number_elements[top]++;
	elements[size] = pair<Domain *,int>(dom,val);
	size++;
}


// restores the elements at the top level and decreases the top of the stack
inline void Basic_Deletion_Stack::Restore (CSP * pb) {
	if (top != -1)	{
		while (number_elements[top] != 0) {
			size--;
			elements[size].first->Unfilter_Value (elements[size].second);
			number_elements[top]--;
		}
		
		top--;
	}
}

#endif
