#include "binary_extension_constraint.h"


// constructs a new binary extension constraint whose all the tuples are forbidden if default_value is false, allowed otherwise
Binary_Extension_Constraint::Binary_Extension_Constraint (vector<Variable *> & var, bool default_value): Extension_Constraint (var) {
	int d1 = scope_var[0]->Get_Domain()->Get_Initial_Size();
	int d2 = scope_var[1]->Get_Domain()->Get_Initial_Size();
		
  relation = new bool * [d1];
  for (int i = 0; i < d1; i++) {
  	relation[i] = new bool [d2];
  	for (int j = 0; j < d2; j++)	relation[i][j] = default_value;
  }
}


// destructor
Binary_Extension_Constraint::~Binary_Extension_Constraint () {
	int d1 = scope_var[0]->Get_Domain()->Get_Initial_Size();
	
  for (int i = 0; i < d1; i++) 	delete [] relation[i];
  
  delete [] relation;
}


//-----------------
// basic functions
//-----------------


// merges the current constraint with the constraint c
void Binary_Extension_Constraint::Merge (Binary_Extension_Constraint * c) {
	if (this != c) {
		int d1 = scope_var[0]->Get_Domain()->Get_Initial_Size();
		int d2 = scope_var[1]->Get_Domain()->Get_Initial_Size();
		
		if (scope_var[0]->Get_Num() == c->scope_var[0]->Get_Num())
			for (int i = 0; i < d1; i++)
				for (int j = 0; j < d2; j++)
					relation[i][j] = (relation[i][j] && c->relation[i][j]);
		else
				for (int i = 0; i < d1; i++)
					for (int j = 0; j < d2; j++)
						relation[i][j] = (relation[i][j] && c->relation[j][i]);
	}
}


// allows the tuple t
void Binary_Extension_Constraint::Allow_Tuple (int * t) {
  relation[t[0]][t[1]] = true;
}


// allows the tuple t
void Binary_Extension_Constraint::Allow_Tuple (int val1, int val2) {
	relation[val1][val2] = true;
}


// forbids the tuple t
void Binary_Extension_Constraint::Forbid_Tuple (int * t) {
	relation[t[0]][t[1]] = false;
}


// allows the tuple t
void Binary_Extension_Constraint::Forbid_Tuple (int val1, int val2) {
	relation[val1][val2] = false;
}


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Binary_Extension_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
	int x,y;
	
	Domain * dx, * dy;
	
	if (scope_var[0]->Get_Num() == var) {
		x = 0;
		y = 1;
		dx = scope_var[0]->Get_Domain();
		dy = scope_var[1]->Get_Domain();
	}
	else 	{
			x = 1;
			y = 0;
			dx = scope_var[1]->Get_Domain();
			dy = scope_var[0]->Get_Domain();
		}

	int dx_size = dx->Get_Size();
		
	int i = 0;
	int v;

  int t [2];
  int * ts;
  bool has_no_support;
  bool modif = false;    // true if a deletion is achieved, false otherwise
	
	
	do {
		v = dx->Get_Remaining_Value (i);
		t[x] = v;
		
		ts = ls->Get_Support (num,x,v);
		
		if ((ts == 0) || (! dy->Is_Present (ts[y]))) {
			has_no_support = true;
			
			if ((ts == 0) || (ls->Continue_From_Scratch()))  t[y] = 0;
			else t[y] = ts[y]+1;
			
			while ((has_no_support) && ((unsigned) t[y] < dy->Get_Initial_Size ()))
        if (dy->Is_Present (t[y])) 
					if (relation[t[0]][t[1]])  has_no_support = false;
					else t[y]++;
				else t[y]++;
	
			if (has_no_support) {
				ds->Add_Element (dx,v);
				dx->Filter_Value (v);
				dx_size--;
				modif = true;
			}
			else  {
					ds->Add_Support (num,x,v,2,ts);
						
					ls->Change_Support (num,x,v,t,2);
					i++;
				}
		}
		else i++;
  }
  while (i < dx_size);


	if (dx->Get_Size () == 0)  weight++;
    
  return modif;
}
