/// \file 
/// \brief Definitions related to the Binary_Extension_Constraint class

#ifndef BINARY_EXTENSION_CONSTRAINT_H
#define BINARY_EXTENSION_CONSTRAINT_H

#include "extension_constraint.h"

class Binary_Extension_Constraint: public Extension_Constraint  {  /// This class implements the binary constraints in extension with boolean matrix
	protected:
		bool ** relation;		///< the relation related to the constraint
		
	public:
		// constructors and destructor
		Binary_Extension_Constraint (vector<Variable *> & var, bool default_value = false); 			///< constructs a binary extension new constraint whose all the tuples are forbidden if default_value is false, allowed otherwise
		~Binary_Extension_Constraint ();																								 					///< destructor
		
		// basic functions
		void Merge (Binary_Extension_Constraint * c);		///< merges the current constraint with the constraint c
		void Allow_Tuple (int * t); 		  			 	 			///< allows the tuple t
		void Allow_Tuple (int val1, int val2); 	  	 		///< allows the tuple t
		void Forbid_Tuple (int * t); 			 	 				 		///< forbids the tuple t
		void Forbid_Tuple (int val1, int val2);  		 		///< allows the tuple t
		virtual bool Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds);	 ///< returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
};

#endif
