/// \file 
/// \brief Definitions related to the Cluster_Heuristic class

#ifndef CLUSTER_HEURISTIC_H
#define CLUSTER_HEURISTIC_H

#include "csp_tree_decomposition.h"

class Cluster_Heuristic  {     /// This class allows to represent the choice of the next cluster
	protected:
		Light_Set<Cluster_Node *> candidates;		///< the set of cluster candidates		

	public:
		// constructor
		Cluster_Heuristic (int n);	  	///< constructs a cluster heuristic which can manege at most n candidates
		

		// basic functions
		Cluster_Heuristic * Duplicate ();							///< duplicates the cluster heuristic
		void Add_Candidate (Cluster_Node * c);					///< adds the cluster c to the candidate set
		void Remove_Candidate (Cluster_Node * c);				///< removes the cluster c from the candidate set
		void Remove_All_Candidates ();									///< removes all the clusters from the candidate set
		Cluster_Node * Choose_Next_Cluster (Tree_Decomposition * td, CSP * pb, bool is_good[]);	///< returns the number of the next cluster 
};


// adds the cluster c to the candidate set
inline void Cluster_Heuristic::Add_Candidate (Cluster_Node * c) {
	candidates.add (c);
}


// removes the cluster c from the candidate set
inline void Cluster_Heuristic::Remove_Candidate (Cluster_Node * c) {
	candidates.remove (c);
}


// removes all the clusters from the candidate set
inline void Cluster_Heuristic::Remove_All_Candidates () {
	candidates.clear ();
}


// duplicates the cluster heuristic
inline Cluster_Heuristic * Cluster_Heuristic::Duplicate (){
	return new Cluster_Heuristic (*this);	
}

#endif
