#include "constant_element_global_constraint.h"

//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a new element constraint with constant val which involves the variable in var
Constant_Element_Global_Constraint::Constant_Element_Global_Constraint (vector<Variable *> & var, int val): Global_Constraint (var) {
  value = val;
  list_value = new int [arity];
  
  Domain * d;
  
  for (unsigned int i = 0; i < arity; i++){
    d = scope_var[i]->Get_Domain();
    unsigned int j = 0;
    while ((j < d->Get_Size()) && (d->Get_Remaining_Real_Value(j) != value))  j++;
    
    if (j == d->Get_Size())  list_value[i] = -1;
    else list_value[i] = j;
  }
}
		
		

// destructor
Constant_Element_Global_Constraint::~Constant_Element_Global_Constraint () {
  delete [] list_value;
}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Constant_Element_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  // we count the number of variables in scope_var which have the value corresponding to the constant value in their current domain
  int nb = 0;
  int index = -1;
  
  for (unsigned int i = 0; (i < arity) && (nb <= 1); i++)
    if (scope_var[i]->Get_Domain()->Is_Present(list_value[i])) {
      nb++;
      index = i;
    }

  int x = Get_Position (var);
  if ((nb == 0) || ((nb == 1) && (index == x))) {
    // there is no index variable or var corresponds to the index variable, so we can remove some values from the variable var
    Domain * dx = scope_var [x]->Get_Domain();

    unsigned int i = 0;
    int v;
    bool modif = false;

    do {
      v = dx->Get_Remaining_Value(i);
      
   		if (list_value[x] != v) {
        ds->Add_Element (dx,v);
        dx->Filter_Value (v);
        
        modif = true;
      }
      else i++;

    }
    while (i < dx->Get_Size());
    
   	if (dx->Get_Size () == 0) weight++;
    
    return modif;
  }
  else return false;  
}
