#include "constant_index_element_global_constraint.h"

//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a new element constraint with constant val which involves the variables in var (the first variable in var represents the index variable)
Constant_Index_Element_Global_Constraint::Constant_Index_Element_Global_Constraint (vector<Variable *> & var, int val): Global_Constraint (var) {
  value = val;
  
  Domain * di = scope_var[0]->Get_Domain();
  
  for (unsigned int v = 0; v < di->Get_Initial_Size(); v++)
    if ((di->Is_Present(v)) && ((di->Get_Real_Value(v) < 0) || (di->Get_Real_Value(v) >= (signed) arity-1)))
      di->Filter_Value (v);   // impossible value
}
		

//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Constant_Index_Element_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  int x = Get_Position (var);
	Domain * dx = scope_var [x]->Get_Domain();
  
  bool modif = false;

  if (x == 0) {
    // we filter the domain of the index variable
    unsigned int i = 0;
    int index;
    int v;
    
    do {
      v = dx->Get_Remaining_Value (i);
      index = dx->Get_Real_Value (v) + 1;

      Domain * dy = scope_var [index]->Get_Domain();
      unsigned int j = 0;
      while ((j < dy->Get_Size()) && (dy->Get_Remaining_Real_Value(j) != value))  j++;
      
      if (j == dy->Get_Size()) {
        // there is no support
        ds->Add_Element (dx,v);
        dx->Filter_Value (v);
        
        modif = true;
      }
      else i++;
    }
    while (i < dx->Get_Size());
  }
  else  {
      // we filter the domain of a variable from the variable list
      if ((scope_var[0]->Get_Domain()->Get_Size() == 1) && (scope_var[0]->Get_Domain()->Get_Remaining_Real_Value(0)+1 == x)) {
        // we delete all the values except one corresponding to the constant value
        int v;
        unsigned int i = 0;
        do {
          v = dx->Get_Remaining_Value (i);

          if (dx->Get_Real_Value (v) == value)  i++;
          else {
              // there is no support
              ds->Add_Element (dx,v);
              dx->Filter_Value (v);
              
              modif = true;
            }
        }
        while (i < dx->Get_Size());
      }
    }

	if (dx->Get_Size () == 0)  weight++;
    
  return modif;
}
