#include "constraint_ac.h"
#include "deletion_stack.h"


//----------------------------
// constructor and destructor
//----------------------------

// constructs an AC algorithm using support structure ls, the constraint selection policy cs and the AC data structure acds which will be achieved in one-pass if one_pass is true, until the fix-point is reached otherwise
Constraint_AC::Constraint_AC (CSP * pb, Multiple_Residue_Support * ls, Constraint_Selection * cs, Filtering_Data_Structure<pair<int,int> > * acds, 
      bool one_pass): AC (ls,cs,one_pass) {
	structure = acds;
	m = pb->Get_M();
	is_selected.resize (m);
	for (unsigned int i = 0; i < m; i++) 	is_selected[i].resize (pb->Get_Constraint(i)->Get_Arity(),false);
}


// destructor
Constraint_AC::~Constraint_AC () {
	delete structure;
}


//----------------
// basic function
//----------------


// achieves, from the variable var (if any), the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
bool Constraint_AC::Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds, int var) {
	unsigned int i;
	unsigned int j;
	int num;
	Multi_Hypergraph * h = pb->Get_Structure();
	Light_Set<Variable *>::iterator iter2;
	Light_List<Edge*>::iterator iter;
	Constraint * c;	
	
	int num_var_iter2;
	
  if ((var == -1) && (A.Get_Size () == 0)) {
		for (i = 0; i < pb->Get_M(); i++){
			c =	pb->Get_Constraint(i);
			j = 0;
			for (iter2 = c->Begin_Var(); iter2 != c->End_Var(); iter2++) {
				structure->Insert (make_pair(i,(*iter2)->Get_Num()));
				is_selected[i][j] = true;
				j++;
			}
		}
		var = -1;
	}
	else {
			for (i = 0; i < pb->Get_M(); i++)
				for (j = 0; j < pb->Get_Constraint(i)->Get_Arity(); j++)
					is_selected[i][j] = false;
			
			if (var == -1)	var = A.Last_Assigned();
			
			for (iter = h->Begin_Edge_List(var); iter != h->End_Edge_List(var); iter++) {
				num = (*iter)->Get_Num();
				c =	pb->Get_Constraint(num);

				if (selection->Is_Eligible (c,var)) {
					j = 0;
					for (iter2 = c->Begin_Var(); iter2 != c->End_Var(); iter2++) {
						num_var_iter2 = (*iter2)->Get_Num();
						if (! A.Is_Assigned (num_var_iter2)) {
							structure->Insert (make_pair(num,num_var_iter2));
							is_selected[num][j] = true;
						}
						j++;
					}
				}
			}
		}

	pair<int,int> p;	

	while (! structure->Is_Empty ()) {
  	p = structure->Current ();
  	structure->Remove();

		c = pb->Get_Constraint (p.first);

    is_selected[p.first][c->Get_Position (p.second)] = onepass;  

		if (c->Has_Specific_Propagator()) {
			bool * modified;
			c->Propagate (pb,A,supports,ds,modified);
			
			int num;
			for (unsigned int i = 0; i < c->Get_Arity(); i++)
				if (modified[i]) {
					if (c->Get_Scope_Variable(i)->Get_Domain()->Get_Size() == 0) {
						structure->Clear ();
						return false;
					}
					
					int num_var = c->Get_Scope_Variable(i)->Get_Num();
					
					for (iter = h->Begin_Edge_List(num_var); iter != h->End_Edge_List(num_var); iter++) {
						num = (*iter)->Get_Num();
						c =	pb->Get_Constraint(num);
						
						if ((num != p.first) && (selection->Is_Eligible (c,var))) {
							j = 0;
							for (iter2 = c->Begin_Var(); iter2 != c->End_Var(); iter2++) {
								num_var_iter2 = (*iter2)->Get_Num();
								if ((num_var_iter2 != num_var) && ((! is_selected[num][j]) && (! A.Is_Assigned (num_var_iter2)))) { 
									is_selected[num][j] = true;
									
									structure->Insert (make_pair(num,num_var_iter2));
								}
								j++;
							}
						}
					}
				}
		}
		else
			if (c->Revise (pb,p.second,supports,ds)) {						
				if (pb->Get_Variable(p.second)->Get_Domain()->Get_Size() == 0) {
					structure->Clear ();
					return false;
				}
				
				for (iter = h->Begin_Edge_List(p.second); iter != h->End_Edge_List(p.second); iter++) {
					num = (*iter)->Get_Num();
					c =	pb->Get_Constraint(num);
					
					if ((num != p.first) && (selection->Is_Eligible (c,var))) {					
						j = 0;
						for (iter2 = c->Begin_Var(); iter2 != c->End_Var(); iter2++) {
							num_var_iter2 = (*iter2)->Get_Num();
							if ((num_var_iter2!= p.second) && ((! is_selected[num][j]) && (! A.Is_Assigned (num_var_iter2)))) { 
								is_selected[num][j] = true;
								
								structure->Insert (make_pair(num,num_var_iter2));
							}
							j++;
						}
					}
				}
		}
  }
  return true;
}
