/// \file 
/// \brief Definitions related to the Constraint_AC class

#ifndef CONSTRAINT_AC_H
#define CONSTRAINT_AC_H

using namespace std;

#include "ac.h"
#include "filtering_data_structure.h"

class Constraint_AC: public AC {        /// This class implements constraint-oriented AC algorithm
	protected:
		Filtering_Data_Structure<pair<int,int> > * structure; 	///< the data structure representing the set of the constraints for which the consistency must be checked
		unsigned int m;											///< the initial number of constraints
		vector<vector<bool> > is_selected;	///< true if the variable x is in the structure, false otherwise

	public:
		// constructor and destructor
		Constraint_AC (CSP * pb, Multiple_Residue_Support * ls, Constraint_Selection * cs, Filtering_Data_Structure<pair<int,int> > * acds, bool one_pass = false);	///< constructs an AC algorithm using support structure ls, the constraint selection policy cs and the AC data structure acds which will be achieved in one-pass if one_pass is true, until the fix-point is reached otherwise
		~Constraint_AC ();						///< destructor

		// basic functions
		virtual bool Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds, int var);	///< achieves, from the variable var (if any), the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
};
#endif

