#include "csp.h"
#include "binary_extension_constraint.h"
#include "negative_nary_extension_constraint.h"
#include "positive_nary_extension_constraint.h"
#include "predicate_constraint.h"
#include "expression.h"
#include <typeinfo>


//----------------------------
// constructor and destructor
//----------------------------

// constructs an empty CSP whose name is pb_name)
CSP::CSP () {
	n = 0;
	m = 0;
	total_value_number = 0;
	h = new Multi_Hypergraph;
}


// destructor
CSP::~CSP () {
	for (unsigned int i = 0; i < m; i++)	delete constraints[i];
	
	for (unsigned int i = 0; i < n; i++)	delete variables[i];
	
	delete h;
}


//-----------------
// basic functions
//-----------------


// adds a new variable v to the CSP 
void CSP::Add_Variable (Variable * v) {
	variables.expand (1);
	variables[n] = v;
	variables[n]->Set_Num (n);
	h->Add_Vertex();
	n++;
	
	total_value_number += v->Get_Domain()->Get_Initial_Size ();
}


// returns a pointer on the variable whose name is label
Variable * CSP::Get_Variable (string label) {
	unsigned int num = 0;
	while ((num < n) && (variables[num]->Get_Name().compare (label) != 0))	num++;

	if (num < n)  return variables[num];
	else return 0;
}


// adds a new constraint c to the CSP and returns the number of the constraint
int CSP::Add_Constraint (Constraint * c) {
	unsigned int a = c->Get_Arity();
	set <unsigned int> elements = c->Get_Scope();
	unsigned int i;
	bool addition = false;
	bool not_found = true;
	
	// we look for a constraint with the same scope as c and whose type is compatible with c's one in order to merge the two constraints
	i = 0;
  if (typeid(*c) == typeid (Negative_Nary_Extension_Constraint))
    while ((i < m) && (not_found)) {
      if ((a == constraints[i]->Get_Arity()) && (typeid(*c) == typeid(*constraints[i])))
        not_found = constraints[i]->Get_Scope() != elements;
      i++;
    }
  else
      while ((i < m) && (not_found)) {
        if ((a == constraints[i]->Get_Arity()) && (typeid(*c) == typeid(*constraints[i])))
          not_found = constraints[i]->Get_Scope() != elements;
        i++;
      }

	if (not_found)  addition = true;
	else {
			i--;
			if (typeid(*constraints[i]) == typeid(*c)) {
				addition = false;
				if (typeid(*constraints[i]) == typeid (Predicate_Constraint))  
          dynamic_cast<Predicate_Constraint*>(constraints[i])->Merge (dynamic_cast<Predicate_Constraint*>(c));
				else if (typeid(*constraints[i]) == typeid (Positive_Nary_Extension_Constraint))  
               dynamic_cast<Positive_Nary_Extension_Constraint*>(constraints[i])->Merge (dynamic_cast<Positive_Nary_Extension_Constraint*>(c));
					   else if (typeid(*constraints[i]) == typeid (Negative_Nary_Extension_Constraint))  
                    dynamic_cast<Negative_Nary_Extension_Constraint*>(constraints[i])->Merge (dynamic_cast<Negative_Nary_Extension_Constraint*>(c));
                else if (typeid(*constraints[i]) == typeid (Binary_Extension_Constraint))  
                       dynamic_cast<Binary_Extension_Constraint*>(constraints[i])->Merge (dynamic_cast<Binary_Extension_Constraint*>(c));
                     else addition = true;
								
				if (!addition)	delete c;		
			}
			else addition = true;
		}
	
	if (addition)	{
		constraints.expand (1);
		constraints[m] = c;
		constraints[m]->Set_Num (m);
	
		h->Graphical_Structure::Add_Edge(elements);
		
		m++;
		return m-1;
	}
	else return i;
}


// replaces the constraint whose number is num by the constraint c (we assume that the two constraints have the same scope)
void CSP::Replace_Constraint (int num, Constraint * c) {
	if ((unsigned) num < m)	{
		delete constraints [num];
		constraints [num] = c;
		c->Set_Num (num);
	}
}

