/// \file 
/// \brief Definitions related to the CSP class

#ifndef CSP_H
#define CSP_H

#include "constraint.h"
#include <multi_hypergraph.h>

using namespace std;

class CSP  {   /// This class allows to represent CSP instances
	protected:
		unsigned int n;													///< number of variables
		unsigned int m;													///< number of constraints
		unsigned int total_value_number;				///< the number of values in all the domains
		Light_Vector<Variable *> variables;     ///< list of the variables of the CSP
		Light_Vector<Constraint *> constraints; ///< list of the constraints of the CSP
		Multi_Hypergraph * h;										///< the constraint hypergraph representing the structure of the CSP
				
	public:
		// constructor and destructor
		CSP ();           ///< constructs an empty CSP
		~CSP ();					///< destructor
		
		// basic functions
		unsigned int Get_N();												 	///< returns the number of variables
		unsigned int Get_M();												 	///< returns the number of constraints
		unsigned int Get_Total_Value_Number();			 	///< returns the number of values in all the domains
		Multi_Hypergraph * Get_Structure();					 	///< returns a pointer on the constraint hypergraph representing the structure of the CSP
		void Add_Variable (Variable * v);  		        ///< adds a new variable v to the CSP (a new memory space is allocated if duplicate is set to true)
		Variable * Get_Variable (int num);					 	///< returns a pointer on the variable number num
		Variable * Get_Variable (string label);				///< returns a pointer on the variable whose name is label
		int Add_Constraint (Constraint * c);          ///< adds a new constraint c to the CSP and returns the number of the constraint
		Constraint * Get_Constraint (int num);			 	///< returns a pointer on the constraint number num
		void Replace_Constraint (int num, Constraint * c);	///< replaces the constraint whose number is num by the constraint c (we assume that the two constraints have the same scope)
};


//-----------------------------
// inline function definitions
//-----------------------------


// returns the number of variables
inline unsigned int CSP::Get_N() {
	return n;
}


// returns the number of constraints
inline unsigned int CSP::Get_M() {
	return m;
}


// returns the number of values in all the domains
inline unsigned int CSP::Get_Total_Value_Number() {
	return total_value_number;
}
	

// returns on a pointer on the constraint hypergraph representing the structure of the CSP
inline Multi_Hypergraph * CSP::Get_Structure() {
	return h;
}


// returns a pointer on the variable number num
inline Variable * CSP::Get_Variable (int num) {
	return variables[num];
}


// returns a pointer on the constraint number num
inline Constraint * CSP::Get_Constraint (int num) {
	return constraints[num];
}
#endif
