#include "csp_child_node.h"


//----------------------------
// constructor and destructor
//----------------------------


// constructs a CSP child node by copying the child node cn and adding the good sets (expressed as lexicographic trees)
CSP_Child_Node::CSP_Child_Node (Child_Node & cn, Cluster_Node * cl, CSP * pb):  Child_Node (cn) {
  proper_variable.expand (cl->Size() - Size());
	proper_variable_number = 0;
	
	for (Light_Vector<vertex>::iterator iter = cl->begin(); iter != cl->end(); iter++)
		if (! Is_Element (*iter))	{
			proper_variable[proper_variable_number] = *iter;
			proper_variable_number++;
		}
		
	tuple = new int [size];
	
	good_number = 0;
}


// destructor
CSP_Child_Node::~CSP_Child_Node () {
  delete [] tuple;
}


//----------------
// good functions
//----------------


// records the assignment A as new good w.r.t. the current child node
void CSP_Child_Node::Record_Good (Assignment & A) {
	for (int i = 0; i < size; i++)  tuple[i] = A.Get_Value (elements[i]);
	
	Light_Vector<int> t (proper_variable_number);
	
	for (int k = 0; k < proper_variable_number; k++)	t[k] = A.Get_Value (proper_variable[k]);

	good_set.Add_Element (tuple,size,t);
}


// adds to the assignment A the good corresponding to the assignment (if it exists)
void CSP_Child_Node::Add_Good_To_Assignment (Assignment & A) {
	int i;

	for (i = 0; i < size; i++) 	tuple[i] = A.Get_Value (elements[i]);

	Light_Vector<int> * t = good_set.Get_Label (tuple,size);
	
	if (t != 0)
		for (int k = 0; k < proper_variable_number; k++)
			if (! A.Is_Assigned (proper_variable[k]))  A.Assign (proper_variable[k],(*t)[k]);
}
