/// \file 
/// \brief Definitions related to the CSP_Child_Node class

#ifndef CSP_CHILD_NODE_H
#define CSP_CHILD_NODE_H

#include <child_node.h>
#include "assignment.h"
#include "labeled_lexicographic_tree.h"

 
class CSP_Child_Node: public Child_Node {     /// This class implements a specialization (based on lexicographic trees) of the Child_Node class for solving CSP instances
	protected:
    Light_Vector<vertex> proper_variable; ///< the proper variable of the cluster (required in order to record goods on the whole cluster)
		int proper_variable_number;						///< the number of proper variables
		int * tuple;													///< the tuple which will be used for checking if an assignment corresponds to a (no)good (without allocating space at each check)
		unsigned long good_number;						///< the number of goods related to the current child node
		Labeled_Lexicographic_Tree<Light_Vector<int>>  good_set;	///< the set of goods related to the child node
	
	public:
		// constructor and destructor
		CSP_Child_Node (Child_Node & cn, Cluster_Node * cl, CSP * pb);  ///< constructs a CSP child node by copying the child node cn and adding the good sets (expressed as lexicographic trees)
		~CSP_Child_Node ();		///< destructor
				
		// good/nogood functions
		virtual bool Is_Good (Assignment & A);				///< returns true if the assignment A corresponds to a good w.r.t. the current child node
		virtual void Record_Good (Assignment & A);		///< records the assignment A as new good w.r.t. the current child node
		virtual void Add_Good_To_Assignment (Assignment & A);	///< adds to the assignment A the good corresponding to the assignment (if it exists)
		virtual void Clear_Goods ();									///< removes all the goods
};


//-----------------------------
// inline function definitions
//-----------------------------


// returns true if the assignment A corresponds to a good w.r.t. the current child node
inline bool CSP_Child_Node::Is_Good (Assignment & A) {
	for (int i = 0; i < size; i++)
		tuple[i] = A.Get_Value (elements[i]);
	
	return good_set.Is_Element (tuple,size);
}


// removes all the goods
inline void CSP_Child_Node::Clear_Goods () {
	good_set.Clear();
}


#endif

