#include "element_global_constraint.h"

//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a new element constraint with the last variable in var as the variable value and whose number is n and which involves the variable in var
Element_Global_Constraint::Element_Global_Constraint (vector<Variable *> & var): Global_Constraint (var) {
  Domain * dv = scope_var[arity-1]->Get_Domain();
  
  list_value = new int * [dv->Get_Initial_Size()];
  
  Domain * d;
  
  for (unsigned int v = 0; v < dv->Get_Initial_Size(); v++)
    if (dv->Is_Present(v))  {
      list_value[v] = new int [arity-1];
      int value = dv->Get_Real_Value(v);
      
      for (unsigned int i = 0; i < arity-1; i++) {
        d = scope_var[i]->Get_Domain();
        unsigned int j = 0;
        
        while ((j < d->Get_Size()) && (d->Get_Remaining_Real_Value(j) != value))  j++;
        
        if (j == d->Get_Size())  list_value[v][i] = -1;
        else list_value[v][i] = j;
      }
    }
    else list_value[v] = 0;
}


// destructor
Element_Global_Constraint::~Element_Global_Constraint () {
  for (unsigned int v = 0; v < scope_var[arity-1]->Get_Domain()->Get_Initial_Size(); v++)
    delete [] list_value[v];
  delete [] list_value;
}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Element_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  unsigned int x = Get_Position (var);
  Domain * dx = scope_var [x]->Get_Domain();
  bool modif = false;
  
  if (x == arity - 1) {
    // var correspond to the variable value
    unsigned int i = 0;
    int v;
    do {
      v = dx->Get_Remaining_Value(i);

      unsigned int j = 0;
      while ((j < arity-1) && (! scope_var[j]->Get_Domain()->Is_Present(list_value[v][j])))  j++;
      
      if (j == arity -1) {
        ds->Add_Element (dx,v);
        dx->Filter_Value (v);
        
        modif = true;
      }
      else i++;
    }
    while (i < dx->Get_Size());
  }
  else if (scope_var[arity-1]->Get_Domain()->Get_Size() == 1) {
         int w = scope_var[arity-1]->Get_Domain()->Get_Remaining_Value(0);
        
         int nb = 0;
         int index = -1;
        
         for (unsigned int i = 0; (i < arity-1) && (nb <= 1); i++)
           if (scope_var[i]->Get_Domain()->Is_Present(list_value[w][i])) {
             nb++;
             index = i;
           }

         if ((nb == 0) || ((nb == 1) && (index == (signed) x))) {
           // there is no index variable or var corresponds to the index variable, so we can remove some values from the variable var
           Domain * dx = scope_var [x]->Get_Domain();

           unsigned int i = 0;
           int v;

           do {
             v = dx->Get_Remaining_Value(i);
            
             if (list_value[w][x] != v) {
               ds->Add_Element (dx,v);
               dx->Filter_Value (v);
              
               modif = true;
             }
             else i++;
           }
           while (i < dx->Get_Size());
         }
       }
  
    
  if (dx->Get_Size () == 0) weight++;
  
  return modif;
}
