#include "equal_weighted_sum_arithmetic_global_constraint.h"


//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a new constraint which involves the variable in var and whose relation compares the sum of the values of the variables in var weighted by weight from w to the constant cst with respect to =
Equal_Weighted_Sum_Arithmetic_Global_Constraint::Equal_Weighted_Sum_Arithmetic_Global_Constraint (vector<Variable *> & var, vector<int> & w, int cst): 
      Weighted_Sum_Arithmetic_Global_Constraint (var,w,cst) {	
	values = new int [arity];
}


// destructor
Equal_Weighted_Sum_Arithmetic_Global_Constraint::~Equal_Weighted_Sum_Arithmetic_Global_Constraint () {
	delete [] values;
}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Equal_Weighted_Sum_Arithmetic_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  unsigned int x = Get_Position (var);
	Domain * dx = scope_var [x]->Get_Domain();
	int dx_size = dx->Get_Size();
	Domain * dy;	
	
	int i = 0;
	int j;
	int real_v;
	
  int * ts;
  bool has_no_support;
  bool record_support;
  bool modif = false;    // true if a deletion is achieved, false otherwise
	bool exist_more_tuples;
	int sum = 0;
	int sum_max = 0;
	int sum_min = 0;
  
  for (unsigned int j = 0; j < arity; j++)
    if (j != x) {
      dy = scope_var[j]->Get_Domain();
      if (weights[j] > 0) {
        sum_min += weights[j] * dy->Get_Real_Value (dy->Get_Min());
        sum_max += weights[j] * dy->Get_Real_Value (dy->Get_Max());
      }
      else {
          sum_min += weights[j] * dy->Get_Real_Value (dy->Get_Max());
          sum_max += weights[j] * dy->Get_Real_Value (dy->Get_Min());
        }
    }

	do {
		values[x] = dx->Get_Remaining_Value (i);
		real_v = dx->Get_Real_Value (values[x]) * weights[x];
		
		ts = ls->Get_Support (num,x,values[x]);
		
		if (ts == 0) {
			// we start with the minimum value of each variable except for var

			if (sum_max + real_v < constant) {
				has_no_support = true;
				record_support = false;
				exist_more_tuples = false;
			}
			else {
     			for (unsigned int j = 0; j < arity; j++)
            if (j != x) {
              dy = scope_var[j]->Get_Domain();
              if (weights[j] > 0)  values[j] = dy->Get_Min();
              else values[j] = dy->Get_Max();							
            }

					record_support = true;
					exist_more_tuples = true;
          sum = sum_min + real_v;
					has_no_support = sum != constant;
				}
		}
		else {
				sum = constant;
				j = 0; 
				while (((unsigned) j < arity) && (scope_var[j]->Get_Domain()->Is_Present(ts[j]))) j++;

				if (j == (signed) arity) {
          // the tuple is still valid
					has_no_support = false;
					record_support = false;
					exist_more_tuples = true;
				}
				else {
						has_no_support = true;
						record_support = true;
						
						if (ls->Continue_From_Scratch()) {
							if (sum_max + real_v < constant) {
								has_no_support = true;
								record_support = false;
								exist_more_tuples = false;
							}
							else {
     							for (unsigned int j = 0; j < arity; j++)
                    if (j != x) {
                      dy = scope_var [j]->Get_Domain();
                      if (weights[j] > 0)  values[j] = dy->Get_Min();
                      else values[j] = dy->Get_Max();
                    }

									record_support = true;
									exist_more_tuples = true;
                  sum = sum_min + real_v;
									has_no_support = sum != constant;
								}
						}
						else {
								sum = constant;
								for (unsigned int j = 0; j < arity; j++)  values[j] = ts[j];
                
								record_support = true;
								exist_more_tuples = true;
								has_no_support = true;
							}
					}
			}
		
		unsigned int current_var_pos = 0;
		
		while ((has_no_support) && (exist_more_tuples)) {
			// we look for the next compatible tuple by changing the value of the variable at position current_var_pos
				
			if (current_var_pos == x) {
				current_var_pos++;
				exist_more_tuples = current_var_pos < arity;
			}
			else {
					sum -= weights[current_var_pos] * scope_var[current_var_pos]->Get_Domain()->Get_Real_Value(values[current_var_pos]);
					
					if (weights[current_var_pos] > 0)  values[current_var_pos] = scope_var[current_var_pos]->Get_Domain()->Next_Value (values[current_var_pos]);
					else values[current_var_pos] = scope_var[current_var_pos]->Get_Domain()->Previous_Value (values[current_var_pos]);

					if ((values[current_var_pos] == -1) || 
              (sum + weights[current_var_pos] * scope_var[current_var_pos]->Get_Domain()->Get_Real_Value (values[current_var_pos]) > constant)) {
						if (weights[current_var_pos] > 0) values[current_var_pos] = scope_var[current_var_pos]->Get_Domain()->Get_Min();
						else values[current_var_pos] = scope_var[current_var_pos]->Get_Domain()->Get_Max();
						sum += weights[current_var_pos] * scope_var[current_var_pos]->Get_Domain()->Get_Real_Value (values[current_var_pos]);
						
						j = current_var_pos-1;
						
						while (j >= 0) {
							if (j != (signed) x) {
								sum -= weights[j] * scope_var[j]->Get_Domain()->Get_Real_Value (values[j]);
								if (weights[j] > 0)  values[j] = scope_var[j]->Get_Domain()->Get_Min();
								else values[j] = scope_var[j]->Get_Domain()->Get_Max();
								sum += weights[j] * scope_var[j]->Get_Domain()->Get_Real_Value (values[j]);
							}
							j--;
						}
						current_var_pos++;
						exist_more_tuples = current_var_pos < arity;
					}
					else {
							sum += weights[current_var_pos] * scope_var[current_var_pos]->Get_Domain()->Get_Real_Value (values[current_var_pos]);
							current_var_pos = 0;
							has_no_support = sum != constant;
						}
				}
		}
		
		if (has_no_support) {
			ds->Add_Element (dx,values[x]);
			dx->Filter_Value (values[x]);
			dx_size--;
			modif = true;
		}
		else {
				if (record_support) {
					ds->Add_Support (num,x,values[x],arity,ts);
					ls->Change_Support (num,x,values[x],values,arity);
				}
				i++;
			}
  }
  while (i < dx_size);
  
	if (dx->Get_Size () == 0)  weight++;
  
  return modif;
}
