#include "expression.h"

//-----------------------------
// constructors and destructor
//-----------------------------


// constructs an empty expression (i.e. whose type is NIL)
Expression::Expression (){
	type = NIL;
	subexpression = 0;
}


// constructs an expression corresponding to a variable or to a value depending on nt
Expression::Expression (node_type nt, int var_val) {
  type = nt;
  varvalarity = var_val;
  subexpression = 0;
}


// constructs an expression corresponding to the operator oper whose operands are given by sub whose size is arity
Expression::Expression (operator_type oper, Expression * sub, int arity) {
	type = OP;
	op = oper;

	switch (op){
		case NEG : varvalarity = 1; break;
		case ABS : varvalarity = 1; break;
		case SQR : varvalarity = 1; break;
		case NOT : varvalarity = 1; break;
		case IF  : varvalarity = 3; break;
		case SUM : varvalarity = arity; break;
		case PROD: varvalarity = arity; break;
		case MIN : varvalarity = arity; break;
		case MAX : varvalarity = arity; break;
		case EQU : varvalarity = arity; break;
		case AND : varvalarity = arity; break;
		case OR  : varvalarity = arity; break;
		case XOR : varvalarity = arity; break;
		case IFF : varvalarity = arity; break;
		case SET : varvalarity = arity; break;
		default  : varvalarity = 2;
	}
	
	subexpression = new Expression [varvalarity];
	
	for (int i = 0; i < varvalarity; i++)		subexpression[i] = sub[i];
}


// constructs an expression corresponding to the operator oper whose operands are given by sub whose size is arity
Expression::Expression (operator_type oper, vector<Expression> & sub) {
	type = OP;
	op = oper;
	
	subexpression = new Expression [sub.size()];
	varvalarity = sub.size();
	
	for (unsigned int i = 0; i < sub.size(); i++)		subexpression[i] = sub[i];
}


// constructs an expression by copying the expression e
Expression::Expression (Expression & e) {
	if (e.type == OP) {
		type = OP;
		op = e.op;

		varvalarity = e.varvalarity;
		subexpression = new Expression [varvalarity];

		for (int i = 0; i < varvalarity; i++)		subexpression[i] = e.subexpression[i];		
	}
	else {
			type = e.type;
			varvalarity = e.varvalarity;
			subexpression = 0;
		}
}


// constructs an expression by copying the expression e
Expression::Expression (const Expression & e) {
	if (e.type == OP) {
		type = OP;
		op = e.op;

		varvalarity = e.varvalarity;
		subexpression = new Expression [varvalarity];

		for (int i = 0; i < varvalarity; i++)  subexpression[i] = e.subexpression[i];		
	}
	else {
			type = e.type;
			varvalarity = e.varvalarity;
			subexpression = 0;
		}
}


// destructor
Expression::~Expression () {
	delete [] subexpression;
}


//----------
// operator
//----------


// operator =
Expression & Expression::operator= (const Expression & e) {
	if (this != &e) {
		delete [] subexpression;
		
		if (e.type == OP) {
			type = OP;
			op = e.op;

			varvalarity = e.varvalarity;
			subexpression = new Expression [varvalarity];

			for (int i = 0; i < varvalarity; i++)   subexpression[i] = e.subexpression[i];		
		}
		else {
				type = e.type;
				varvalarity = e.varvalarity;
				subexpression = 0;
			}
	}
	return *this;
}


// operator ==
bool Expression::operator== (const Expression & e) {
	if (this == &e)  return true;
	else if (type == OP)
         if (type == e.type) 
           if ((op == e.op) && (varvalarity == e.varvalarity)) {
             int i;
             if ((op == MAX) || (op == MIN) || (op == ADD) || (op == SUM) || (op == MUL) || (op == PROD) || (op == AND) || (op == OR) || (op == EQ) || 
                 (op == EQU) || (op == NE) || (op == IFF) || (op == XOR) || (op == DIST) || (op == SET))  {
               // the operator is commutative
               int j;
               i = 0; 
               do {
                 j = 0;
                 while ((j < varvalarity) && (subexpression[i] != e.subexpression[j]))  j++;
                   
                 if (j < varvalarity)  i++;
               }
               while ((i < varvalarity) && (j < varvalarity));
             }
             else for (i = 0; (i < varvalarity) && (subexpression[i] == e.subexpression[i]); i++);
             
             return (i == varvalarity);	
           }
         else return false;
      else return false;    
    else return (type == e.type) && (varvalarity == e.varvalarity);
}


//-----------------
// basic functions
//-----------------


// modifies the expression by providing effective value or changing the variable name for the variables of the current expression
void Expression::Change_Variable (vector<int> effective_parameters, vector<node_type> effective_parameter_type) {
	if (type == VAR) {
		type = effective_parameter_type[varvalarity];
		varvalarity = effective_parameters[varvalarity];
	}
	else if (type == OP)
         for (int i = 0; i < varvalarity; i++)		subexpression[i].Change_Variable(effective_parameters,effective_parameter_type);
}
