/// \file 
/// \brief Definitions related to the Filtering_Data_Structure class

#ifndef FILTERING_DATA_STRUCTURE_H
#define FILTERING_DATA_STRUCTURE_H

using namespace std;

#include <queue>

template<class T>
class Filtering_Data_Structure {  /// This class implements the queue representing the set of the constraints for which the consistency must be checked 
	protected:
		queue<T> elements;    ///< the elements which defines the set of constraints

	public:	
		// basic functions
		bool Is_Empty();	 		 ///< returns true if the structure is empty, false otherwise
		void Insert (T elt);	 ///< adds an element in the structure
		void Remove ();				 ///< removes the current element of the structure
		T Current ();					 ///< returns the current element of the structure
		void Clear (); 				 ///< removes all the elements of the structure
};


// returns true if the structure is empty, false otherwise
template<class T> inline bool Filtering_Data_Structure<T>::Is_Empty() {
	return elements.empty();
}


// adds an element in the structure
template<class T> inline void Filtering_Data_Structure<T>::Insert (T elt) {
	elements.push(elt);
}


// removes the current element of the structure
template<class T> inline void Filtering_Data_Structure<T>::Remove () {
	elements.pop();
}


// returns the current element of the structure
template<class T> inline T Filtering_Data_Structure<T>::Current () {
	return elements.front ();
}


// removes all the elements of the structure
template<class T> inline void Filtering_Data_Structure<T>::Clear () {
	while  (! elements.empty())		elements.pop();
}

#endif
