#include "in_weighted_sum_arithmetic_global_constraint.h"


//--------------
// constructors 
//--------------


// constructs a new constraint which involves the variable in var and which checks whether the weighted sum (weighted by weight from w) of the values of the variables in var in the interval [cst1, cst2]
In_Weighted_Sum_Arithmetic_Global_Constraint::In_Weighted_Sum_Arithmetic_Global_Constraint (vector<Variable *> & var, vector<int> & w, int cst1, int cst2):
   Weighted_Sum_Arithmetic_Global_Constraint (var,w,cst1) {	
  constant2 = cst2;
}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool In_Weighted_Sum_Arithmetic_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
	int sum_min = 0;
	int sum_max = 0;
  unsigned int x = Get_Position (var);
  Domain * dx = scope_var [x]->Get_Domain();
	Domain * dy;

	for (unsigned int i = 0; i < arity; i++)
		if (i != x) {
			dy = scope_var [i]->Get_Domain();
	
      if (weights[i] > 0) {
        sum_min += weights[i] * dy->Get_Real_Value (dy->Get_Min());
        sum_max += weights[i] * dy->Get_Real_Value (dy->Get_Max());
      }
      else {
          sum_min += weights[i] * dy->Get_Real_Value (dy->Get_Max());
          sum_max += weights[i] * dy->Get_Real_Value (dy->Get_Min());
        }
		}

	bool modif = false;
	int v = dx->Get_Min();
	
	while ((v != -1) && (sum_max + dx->Get_Real_Value(v) * weights[x] < constant)) {
		ds->Add_Element (dx,v);
		dx->Filter_Value (v);
		modif = true;
		
		v = dx->Next_Value(v);
	}

	v = dx->Get_Max();
	
	while ((v != -1) && (sum_min + dx->Get_Real_Value(v) * weights[x] > constant2)) {
		ds->Add_Element (dx,v);
		dx->Filter_Value (v);
			
		modif = true;

		v = dx->Previous_Value(v);
	}
  
	if (dx->Get_Size () == 0)   weight++;
	
  return modif;
}
