#include "index_element_global_constraint.h"

//----------------------------
// constructor and destructor
//----------------------------


// constructs a new element constraint with the first variable in var represents the index variable and the last variable in var as the variable value, and which involves the variable in var
Index_Element_Global_Constraint::Index_Element_Global_Constraint (vector<Variable *> & var): Global_Constraint (var) {
  Domain * di = scope_var[0]->Get_Domain();
  
  for (unsigned int v = 0; v < di->Get_Initial_Size(); v++)
    if ((di->Is_Present(v)) && ((di->Get_Real_Value(v) < 0) || (di->Get_Real_Value(v) >= (signed) arity-1)))
      di->Filter_Value (v);   // impossible value
    
  Domain * dv = scope_var[arity-1]->Get_Domain();
  
  list_value = new int * [dv->Get_Initial_Size()];
  
  Domain * d;
  
  for (unsigned int v = 0; v < dv->Get_Initial_Size(); v++)
    if (dv->Is_Present(v)) {
      int value = dv->Get_Real_Value(v);
     
      list_value[v] = new int [arity-2];
      
      for (int i = 0; i < (signed)arity-2; i++) {
        d = scope_var[i+1]->Get_Domain();
        unsigned int j = 0;
        while ((j < d->Get_Size()) && (d->Get_Remaining_Real_Value(j) != value))  j++;
        
        if (j == d->Get_Size())  list_value[v][i] = -1;
        else list_value[v][i] = j;
      }
    }
    else list_value[v] = 0;
}


// destructor
Index_Element_Global_Constraint::~Index_Element_Global_Constraint () {
  for (unsigned int v = 0; v < scope_var[arity-1]->Get_Domain()->Get_Initial_Size(); v++)
    delete [] list_value[v];
  delete [] list_value;
}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Index_Element_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  unsigned int x = Get_Position (var);
	Domain * dx = scope_var [x]->Get_Domain();
  
  bool modif = false;

  if (x == 0) {
    // we filter the domain of the index variable
    unsigned int i = 0;
    int index;
    int v;
    Domain * dv = scope_var[arity-1]->Get_Domain();
    
    do {
      v = dx->Get_Remaining_Value (i);
      index = dx->Get_Real_Value (v);

      Domain * dy = scope_var [index+1]->Get_Domain();
      
      unsigned int j = 0;
      while ((j < dv->Get_Size()) && ((list_value[dv->Get_Remaining_Value(j)][index] == -1)||(! dy->Is_Present(list_value[dv->Get_Remaining_Value(j)][index]))))
        j++;
      
      if (j == dv->Get_Size()) {
        // there is no support
        ds->Add_Element (dx,v);
        dx->Filter_Value (v);
        
        modif = true;
      }
      else i++;
    }
    while (i < dx->Get_Size());
  }
  else if (x == arity - 1) {
         // we filter the domain of the value variable
         unsigned int i = 0;
         int v;
         do {
           v = dx->Get_Remaining_Value(i);

           unsigned int j = 0;
           while ((j < arity-2) && ((list_value[v][j] == -1) || (! scope_var[j+1]->Get_Domain()->Is_Present(list_value[v][j]))))    j++;
            
           if (j == arity -2) {
             ds->Add_Element (dx,v);
             dx->Filter_Value (v);
             
             modif = true;
           }
           else i++;
         }
         while (i < dx->Get_Size());
       }
       else if ((scope_var[0]->Get_Domain()->Get_Size() == 1) && (scope_var[0]->Get_Domain()->Get_Remaining_Real_Value(0)+1 == (signed) x)) {
              // we filter the domain of a variable from the variable list
              // we delete all the values except one corresponding to the constant value
              int v;
              unsigned int i = 0;
              unsigned int j;
              Domain * dv = scope_var[arity-1]->Get_Domain();
              int real_v;
              
              do {
                v = dx->Get_Remaining_Value (i);
                real_v = dx->Get_Real_Value (v);

                j = 0;
                while ((j < dv->Get_Size()) && (dv->Get_Remaining_Real_Value(j) != real_v))   j++;
                  
                if (j < dv->Get_Size())  i++;
                else {
                    // there is no support
                    ds->Add_Element (dx,v);
                    dx->Filter_Value (v);
                    
                    modif = true;
                  }
              }
              while (i < dx->Get_Size());
            }

	if (dx->Get_Size () == 0)  weight++;
    
  return modif;
}
