#include "less_equal_weighted_sum_arithmetic_global_constraint.h"


//-------------
// constructor 
//-------------


// constructs a new constraint which involves the variable in var and whose relation compares the sum of the values of the variables in var weighted by weight from w to the constant cst with respect to <=
Less_Equal_Weighted_Sum_Arithmetic_Global_Constraint::Less_Equal_Weighted_Sum_Arithmetic_Global_Constraint (vector<Variable *> & var, vector<int> & w, int cst): 
           Weighted_Sum_Arithmetic_Global_Constraint (var,w,cst) {}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Less_Equal_Weighted_Sum_Arithmetic_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
	int sum = 0;
	Domain * dy;

	for (unsigned int i = 0; i < arity; i++)
		if (scope_var[i]->Get_Num() != var) {
			dy = scope_var [i]->Get_Domain();
			if (weights[i] > 0)  sum += dy->Get_Real_Value (dy->Get_Min()) * weights[i];
			else sum += dy->Get_Real_Value (dy->Get_Max()) * weights[i];
		}

  int x = Get_Position (var);
	Domain * dx = scope_var [x]->Get_Domain();
	int dx_size = dx->Get_Size();
	
	bool modif = false;
			
	if (weights[x] > 0) {
		if (sum+dx->Get_Real_Value (dx->Get_Max())*weights[x] > constant) {
			int i = dx->Get_Max();
			modif = true;
      
			do {
				ds->Add_Element (dx,i);
				dx->Filter_Value (i);
				dx_size--;
					
				i = dx->Previous_Value (i);
			}
			while ((i != -1) && (sum + dx->Get_Real_Value (i) * weights[x] > constant));
			
			if (dx->Get_Size () == 0)   weight++;
		}
	}
	else if (sum+dx->Get_Real_Value (dx->Get_Min())*weights[x] > constant) {
         int i = dx->Get_Min();
         modif = true;
          
         do {
           ds->Add_Element (dx,i);
           dx->Filter_Value (i);
           dx_size--;
             
           i = dx->Next_Value (i);
         }
         while ((i != -1) && (sum + dx->Get_Real_Value (i) * weights[x] > constant));
          
         if (dx->Get_Size () == 0) 	weight++;
       }
			
  return modif;
}
