/// \file 
/// \brief Definitions related to the Load_XCSP3Callbacks class

#ifndef LOAD_XCSP3PCALLBACKS_H
#define LOAD_XCSP3PCALLBACKS_H

#include <XCSP3CoreCallbacks.h>
#include <XCSP3Variable.h>
#include "csp.h"

using XCSP3Core::XCSP3CoreCallbacks;
using XCSP3Core::InstanceType;
using XCSP3Core::XVariable;
using XCSP3Core::OrderType;
using XCSP3Core::XTransition;
using XCSP3Core::XCondition;
using XCSP3Core::XInterval;
using XCSP3Core::RankType;
using XCSP3Core::ExpressionObjective;

class Load_XCSP3Callbacks : public XCSP3CoreCallbacks {    /// This class makes the link between the current library and the XCSP3 parser from xcsp.org
	protected:
		CSP * pb;                           ///< the CSP which is built from the XCSP3 file
		vector<vector<int>> last_tuples;    ///< the last tuple list (if any)

  public:
		// constructor 
		Load_XCSP3Callbacks(CSP * pb_ref);			///< constructor

		virtual void beginConstraints() override {};           ///< performs the work required when the beginning of the constraint section is reached

		virtual void endConstraints() override {};             ///< performs the work required when the end of the constraint section is reached

		virtual void beginBlock(string classes) override  {};   ///< performs the work required when the beginning of a block is reached

		virtual void endBlock() override {};                   ///< performs the work required when the end of a block is reached

		virtual void beginSlide(string id, bool circular) override {};   ///< performs the work required when the beginning of a slide is reached

		virtual void endSlide() override {};                   ///< performs the work required when the beginning of a slide is reached

		virtual void beginObjectives() override {};            ///< performs the work required when the beginning of the objective section is reached

		virtual void endObjectives() override {};              ///< performs the work required when the end of the objective section is reached


		virtual void buildVariableInteger(string id, int minValue, int maxValue) override;    ///< creates an integer variable whose domain contains the integer values between minValue and Maxvalue

		virtual void buildVariableInteger(string id, vector<int> &values) override;           ///< creates an integer variable whose domain contains the integer values in vector values

		virtual void buildConstraintExtension(string id, vector<XVariable *> list, vector<vector<int> > &tuples, bool support, bool hasStar) override;    ///< creates an extension constraint where list is the scope, tuples the list of tuples which are allowad if support is set to true, forbidden otherwise (hasStar is true if * is allowed in tuples)

		virtual void buildConstraintExtension(string id, XVariable *variable, vector<int> &tuples, bool support, bool hasStar) override;    ///< creates an extension constraint where variable is the single variable in the scope, tuples the list of tuples which are allowad if support is set to true, forbidden otherwise (hasStar is true if * is allowed in tuples)

		virtual void buildConstraintExtensionAs(string id, vector<XVariable *> list, bool support, bool hasStar) override;    ///< creates an extension constraint where list is the scope, the list of tuples which are allowad if support is set to true, forbidden otherwise, is the same as the previous extension constraint (hasStar is true if * is allowed in tuples)

		virtual void buildConstraintIntension(string id, string expr) override;   ///< creates an intension constraint defined by the expression expr

		virtual void buildConstraintPrimitive(string id, OrderType op, XVariable *x, int k, XVariable *y) override;     ///< creates an intension constraint defined by the expression x + k op y


		//~ virtual void buildConstraintRegular(string id, vector<XVariable *> &list, string st, string final, vector<XTransition> &transitions) override;

		//~ virtual void buildConstraintMDD(string id, vector<XVariable *> &list, vector<XTransition> &transitions) override;

		virtual void buildConstraintAlldifferent(string id, vector<XVariable *> &list) override;      ///< creates an all-diff global constraint

		//~ virtual void buildConstraintAlldifferentExcept(string id, vector<XVariable *> &list, vector<int> &except) override;

		//~ virtual void buildConstraintAlldifferentList(string id, vector<vector<XVariable *> > &lists) override;

		//~ virtual void buildConstraintAlldifferentMatrix(string id, vector<vector<XVariable *> > &matrix) override;

		//~ virtual void buildConstraintAllEqual(string id, vector<XVariable *> &list) override;    ///< creates an all-equal global constraint whose scope is provided by list

		//~ virtual void buildConstraintNotAllEqual(string id, vector<XVariable *> &list) override;

		//~ virtual void buildConstraintOrdered(string id, vector<XVariable *> &list, OrderType order) override;

		//~ virtual void buildConstraintLex(string id, vector<vector<XVariable *> > &lists, OrderType order) override;

		//~ virtual void buildConstraintLexMatrix(string id, vector<vector<XVariable *> > &matrix, OrderType order) override;

		virtual void buildConstraintSum(string id, vector<XVariable *> &list, vector<int> &coeffs, XCondition &cond) override;    ///< creates a weighted sum global constraint whose scope is provided by list and for which coeffs corresponds to the list of coefficients and cond is the considered condition

		virtual void buildConstraintSum(string id, vector<XVariable *> &list, XCondition &cond) override;     ///< creates a sum global constraint whose scope is provided by list and for which cond is the considered condition

		//~ virtual void buildConstraintAtMost(string id, vector<XVariable *> &list, int value, int k) override;

		//~ virtual void buildConstraintAtLeast(string id, vector<XVariable *> &list, int value, int k) override;

		//~ virtual void buildConstraintExactlyK(string id, vector<XVariable *> &list, int value, int k) override;

		//~ virtual void buildConstraintAmong(string id, vector<XVariable *> &list, vector<int> &values, int k) override;

		//~ virtual void buildConstraintExactlyVariable(string id, vector<XVariable *> &list, int value, XVariable *x) override;

		//~ virtual void buildConstraintCount(string id, vector<XVariable *> &list, vector<int> &values, XCondition &xc) override;

		//~ virtual void buildConstraintCount(string id, vector<XVariable *> &list, vector<XVariable *> &values, XCondition &xc) override;

		//~ virtual void buildConstraintNValues(string id, vector<XVariable *> &list, vector<int> &except, XCondition &xc) override;

		//~ virtual void buildConstraintNValues(string id, vector<XVariable *> &list, XCondition &xc) override;

		//~ virtual void buildConstraintCardinality(string id, vector<XVariable *> &list, vector<int> values, vector<int> &occurs, bool closed) override;

		//~ virtual void buildConstraintCardinality(string id, vector<XVariable *> &list, vector<int> values, vector<XVariable *> &occurs, bool closed) override;

		//~ virtual void buildConstraintCardinality(string id, vector<XVariable *> &list, vector<int> values, vector<XInterval> &occurs, bool closed) override;

		//~ virtual void buildConstraintCardinality(string id, vector<XVariable *> &list, vector<XVariable *> values, vector<int> &occurs, bool closed) override;

		//~ virtual void buildConstraintCardinality(string id, vector<XVariable *> &list, vector<XVariable *> values, vector<XVariable *> &occurs, bool closed) override;

		//~ virtual void buildConstraintCardinality(string id, vector<XVariable *> &list, vector<XVariable *> values, vector<XInterval> &occurs, bool closed) override;

		//~ virtual void buildConstraintMinimum(string id, vector<XVariable *> &list, XCondition &xc) override;

		//~ virtual void buildConstraintMinimum(string id, vector<XVariable *> &list, XVariable *index, int startIndex, RankType rank, XCondition &xc) override;

		//~ virtual void buildConstraintMaximum(string id, vector<XVariable *> &list, XCondition &xc) override;

		//~ virtual void buildConstraintMaximum(string id, vector<XVariable *> &list, XVariable *index, int startIndex, RankType rank, XCondition &xc) override;

		virtual void buildConstraintElement(string id, vector<XVariable *> &list, int value) override;            ///< creates a element global constraint whose scope is provided by list with a constant value provided by value

		virtual void buildConstraintElement(string id, vector<XVariable *> &list, XVariable *value) override;     ///< creates a element global constraint whose scope is provided by list with a variable value provided by value

		virtual void buildConstraintElement(string id, vector<XVariable *> &list, int startIndex, XVariable *index, RankType rank, int value) override;   ///< creates a element global constraint whose scope is provided by list with a constant value provided by value and index as a index variable 

		virtual void buildConstraintElement(string id, vector<XVariable *> &list, int startIndex, XVariable *index, RankType rank, XVariable *value) override;    ///< creates a element global constraint whose scope is provided by list with a variable value provided by value and index as a index variable 

		//~ virtual void buildConstraintChannel(string id, vector<XVariable *> &list, int startIndex) override;

		//~ virtual void buildConstraintChannel(string id, vector<XVariable *> &list1, int startIndex1, vector<XVariable *> &list2,	int startIndex2) override;

		//~ virtual void buildConstraintChannel(string id, vector<XVariable *> &list, int startIndex, XVariable *value) override;

		//~ virtual void buildConstraintStretch(string id, vector<XVariable *> &list, vector<int> &values, vector<XInterval> &widths) override;

		//~ virtual void buildConstraintStretch(string id, vector<XVariable *> &list, vector<int> &values, vector<XInterval> &widths, vector<vector<int> > &patterns) override;

		//~ virtual void buildConstraintNoOverlap(string id, vector<XVariable *> &origins, vector<int> &lengths, bool zeroIgnored) override;

		//~ virtual void buildConstraintNoOverlap(string id, vector<XVariable *> &origins, vector<XVariable *> &lengths, bool zeroIgnored) override;

		//~ virtual void buildConstraintNoOverlap(string id, vector<vector<XVariable *> >&origins, vector<vector<int> >&lengths, bool zeroIgnored) override;

		//~ virtual void buildConstraintNoOverlap(string id, vector<vector<XVariable *> >&origins, vector<vector<XVariable *> >&lengths, bool zeroIgnored) override;

		//~ virtual void buildConstraintInstantiation(string id, vector<XVariable *> &list, vector<int> &values) override;

		//~ virtual void buildObjectiveMinimizeExpression(string expr) override;

		//~ virtual void buildObjectiveMaximizeExpression(string expr) override;


		//~ virtual void buildObjectiveMinimizeVariable(XVariable *x) override;


		//~ virtual void buildObjectiveMaximizeVariable(XVariable *x) override;


		//~ virtual void buildObjectiveMinimize(ExpressionObjective type, vector<XVariable *> &list, vector<int> &coefs) override;


		//~ virtual void buildObjectiveMaximize(ExpressionObjective type, vector<XVariable *> &list, vector<int> &coefs) override;


		//~ virtual void buildObjectiveMinimize(ExpressionObjective type, vector<XVariable *> &list) override;


		//~ virtual void buildObjectiveMaximize(ExpressionObjective type, vector<XVariable *> &list) override;


};



#endif
