/// \file 
/// \brief Definitions related to the Multiple_Residue_Support class

#ifndef MULTIPLE_RESIDUE_SUPPORT
#define MULTIPLE_RESIDUE_SUPPORT

class CSP;


class Multiple_Residue_Support {     /// This class implements the multiple residue supports

  protected:
		int **** support;		///< support[num][x][v] is the support (i.e. a tuple) of the variable x for the value v w.r.t. the constraint num
		unsigned int m;			///< the initial number of constraints
		CSP * pb;									///< the CSP to which the structure is related 

  
	public:
		// constuctor and destructor
		Multiple_Residue_Support (CSP * pb_init);															///< constructs an empty support for each variable x for each value v w.r.t. each constraint num
		~Multiple_Residue_Support ();																					///< destructor
	
		// basic functions
		void Reset_Support (int num, int pos, int val) {}; 											///< resets the support of the variable at the position pos in the constraint scope for the value v w.r.t. the constraint num
		void Change_Support (int num, int pos, int val, int * t, int arity); 	///< sets the support of the variable at the position pos in the constraint scope for the value v w.r.t. the constraint num to the tuple t
		int * Get_Support (int num, int pos, int val);  			 								///< returns a reference to the support of the variable at the position pos in the constraint scope for the value v w.r.t. the constraint num
		bool Continue_From_Scratch (); 																			  ///< returns true if the search for a support must continue from scratch, false if it can continue from the next tuple
};


//-----------------------------
// inline function definitions
//-----------------------------


// sets the support of the variable at the position pos in the constraint scope for the value v w.r.t. the constraint num to the tuple t
inline void Multiple_Residue_Support::Change_Support (int num, int pos, int val, int * t, int arity) {
	for (int j = 0; j < arity; j++) {
		if (support[num][j][t[j]] == 0)
			support[num][j][t[j]] = new int [arity];
		
		for (int i = 0; i < arity; i++) 	support[num][j][t[j]][i] = t[i];
	}
}


// returns a reference to the support of the variable at the position pos in the constraint scope for the value v w.r.t. the constraint num
inline int * Multiple_Residue_Support::Get_Support (int num, int pos, int val) {
	if (((unsigned) num < m) && (support[num][pos][val] != 0))	return support[num][pos][val];
	else return 0;
}


// returns true if the search for a support must continue from scratch, false if it can continue from the next tuple
inline bool Multiple_Residue_Support::Continue_From_Scratch () {
	return true;
}


#endif
