#include "negative_nary_extension_constraint.h"
#include "avl_iterator.h"

// constructs a new constraint which involves the variable in var whose all the tuples are allowed
Negative_Nary_Extension_Constraint::Negative_Nary_Extension_Constraint (vector<Variable *> & var): Nary_Extension_Constraint (var) {}


//-----------------
// basic functions
//-----------------


// merges the current constraint with the constraint c

void Negative_Nary_Extension_Constraint::Merge (Negative_Nary_Extension_Constraint * c) {
	if (this != c) {
		int pos[arity];
		for (unsigned int i = 0; i < arity; i++)
			pos[i] = c->Get_Position (scope_var[i]->Get_Num());

		int size = scope_var[0]->Get_Domain()->Get_Initial_Size();
		int t [arity];
		for (int i = 0; i < size; i++) {
			for (AVL::iterator iter = c->relation->begin(pos[0],i); iter != c->relation->end(pos[0]); iter++) {
				for (unsigned int j = 0; j < arity; j++)
					t[j] = (*iter)[pos[j]];
					
				if (! relation->Is_Present (t))  relation->Add_Element (t);
			}
		}
	}
}


// forbids the tuple t
void Negative_Nary_Extension_Constraint::Forbid_Tuple (int * t) {
	relation->Add_Element (t);
}


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Negative_Nary_Extension_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  int x = Get_Position (var);
  
	Domain * dx = scope_var [x]->Get_Domain();
	int dx_size = dx->Get_Size();
	
	int i = 0;
	int v;
	
  Tuple t (arity);
  int * ts;
  bool has_no_support;
  bool record_support;
  bool modif = false;    // true if a deletion is achieved, false otherwise
	bool exist_more_tuples;
	
	do {    
		v = dx->Get_Remaining_Value (i);
		
		ts = ls->Get_Support (num,x,v);
		
		if (ts == 0)  {
			for (unsigned int j = 0; j < arity; j++)	t[j] = -1;
      
			t[x] = v;
			t.Complete (scope_var,x);
			has_no_support = true;
			record_support = true;
			exist_more_tuples = true;
		}
		else {
				for (unsigned int j = 0; j < arity; j++)  t[j] = ts[j];
					
				if (t.Is_Valid (scope_var)) {
					has_no_support = false;
					record_support = false;
					exist_more_tuples = true;
				}
				else {
						has_no_support = true;
						record_support = true;
						
						if (ls->Continue_From_Scratch()) {
							for (unsigned int j = 0; j < arity; j++)  t[j] = -1;
              
							t[x] = v;
							t.Complete (scope_var,x);
							exist_more_tuples = true;							
						}
						else exist_more_tuples = t.Next_Compatible_Valid (scope_var,x);
					}
			}
		
		while ((has_no_support) && (exist_more_tuples)) {
			if (! relation->Is_Present (t.Get_Values()))	has_no_support = false;
			else exist_more_tuples = t.Next_Valid (scope_var,x);
		}
		
		if (has_no_support) {
			ds->Add_Element (dx,v);
			dx->Filter_Value (v);
			dx_size--;
			modif = true;
		}
		else  {
				if (record_support) {
					ds->Add_Support (num,x,v,arity,ts);
					ls->Change_Support (num,x,v,t.Get_Values(),arity);
				}
				i++;
			}
  }
  while (i < dx_size);

	if (dx->Get_Size () == 0)  weight++;
    
  return modif;
}
