#include "nogood_base_ac.h"


//----------------------------
// constructor and destructor
//----------------------------

// constructs an AC algorithm using support structure ls, the constraint selection policy cs and the AC data structure acds which will be achieved in one-pass if one_pass is true, until the fix-point is reached otherwise
Nogood_Base_AC::Nogood_Base_AC (CSP * ref_pb, Multiple_Residue_Support * ls, Constraint_Selection * cs, bool one_pass): AC (ls,cs,one_pass) {	
	pb = ref_pb;
	int n = pb->Get_N();
	nogood_base = new Light_List <element *> * [n];
	for (int i = 0; i < n; i++)
		nogood_base[i] = new Light_List <element *> [pb->Get_Variable(i)->Get_Domain()->Get_Initial_Size()];
}


// destructor
Nogood_Base_AC::~Nogood_Base_AC () {
	int n = pb->Get_N();
	
	for (int i = 0; i < n; i++) {
		for (unsigned int j = 0; j < pb->Get_Variable(i)->Get_Domain()->Get_Initial_Size(); j++) {
			for (Light_List<element*>::iterator iter = nogood_base[i][j].begin (); iter != nogood_base[i][j].end (); iter++) {
				if ((*iter)->nogood[(*iter)->watch1].first < (*iter)->nogood[(*iter)->watch2].first) {
					if ((*iter)->nogood[(*iter)->watch2].first == i) {
						delete [] (*iter)->nogood;
						delete (*iter);
					}
				}
				else if ((*iter)->nogood[(*iter)->watch1].first == i) {
               delete [] (*iter)->nogood;
               delete (*iter);
             }
			}
			nogood_base[i][j].clear();
		}
		delete [] nogood_base[i];
	}
	delete [] nogood_base;
}


//-----------------
// basic functions
//-----------------


// adds the nogood ng involving size variables into the nogood base
void Nogood_Base_AC::Add_Nogood (pair <int,int> * ng, int size) {
	element * e = new element;

	e->nogood = new pair <int,int> [size];
	for (int i = 0; i < size; i++)  e->nogood[i] = ng[i];

	e->size = size;
	e->watch1 = 0;
	e->watch2 = size-1;

	nogood_base[ng[e->watch1].first][ng[e->watch1].second].push_front (e);
	nogood_base[ng[e->watch2].first][ng[e->watch2].second].push_front (e);
}




// adds the nogood ng involving size variables into the nogood base and selects the watch literals depending on the assignment A
void Nogood_Base_AC::Add_Nogood (pair <int,int> * ng, int size, Assignment & A) {
	element * e = new element;

	e->nogood = new pair <int,int> [size];
	for (int i = 0; i < size; i++)  e->nogood[i] = ng[i];

	e->size = size;
	
	if (A.Get_Position(ng[0].first) < A.Get_Position(ng[1].first)) {
		e->watch1 = 0;
		e->watch2 = 1;
	}
	else {
			e->watch1 = 1;
			e->watch2 = 0;
		}

	for (int i = 2; i < size; i++)
		if (A.Get_Position(ng[i].first) > A.Get_Position(ng[e->watch2].first)) {
			e->watch1 = e->watch2;
			e->watch2 = i;
		}
		else if (A.Get_Position(ng[i].first) > A.Get_Position(ng[e->watch1].first))
           e->watch1 = i;
	
	nogood_base[ng[e->watch1].first][ng[e->watch1].second].push_front (e);
	nogood_base[ng[e->watch2].first][ng[e->watch2].second].push_front (e);
}


// clears the nogood base
void Nogood_Base_AC::Clear () {
	int n = pb->Get_N();
	
	for (int i = 0; i < n; i++) {
		int size_domain = pb->Get_Variable(i)->Get_Domain()->Get_Initial_Size();
		for (int j = 0; j < size_domain; j++) {
			for (Light_List<element*>::iterator iter = nogood_base[i][j].begin (); iter != nogood_base[i][j].end (); iter++) {
				if ((*iter)->nogood[(*iter)->watch1].first < (*iter)->nogood[(*iter)->watch2].first) {
					if ((*iter)->nogood[(*iter)->watch2].first == i) {
						delete [] (*iter)->nogood;
						delete (*iter);
					}
				}
				else if ((*iter)->nogood[(*iter)->watch1].first == i) {
               delete [] (*iter)->nogood;
               delete (*iter);
             }
			}
			nogood_base[i][j].clear();
		}
	}
}
