/// \file 
/// \brief Definitions related to the Nogood_Base_AC class

#ifndef NOGOOD_BASE_AC_H
#define NOGOOD_BASE_AC_H

#include "ac.h"

class Nogood_Base_AC: public AC {       /// This class allows to represent AC algorithm with nogood base
	protected:
		typedef struct elt {          /// allows to represent nogoods
			pair <int,int> * nogood;    ///< the pairs <variable,value> which form the nogood
			int size;                   ///< the number of pairs (i.e. the size of the nogood)
			int watch1;                 ///< the first watch literal
			int watch2;                 ///< the second watch literal
		} element;                    ///< allows to represent nogoods with watched literals
		Light_List <element *> ** nogood_base;			///< the nogood base
		CSP * pb;																		///< the CSP to which the nogood base is related

	public:
		// constructor and destructor
		Nogood_Base_AC (CSP * ref_pb, Multiple_Residue_Support * ls, Constraint_Selection * cs, bool one_pass = false);	///< constructs an AC algorithm using support structure ls, the constraint selection policy cs and the AC data structure acds which will be achieved in one-pass if one_pass is true, until the fix-point is reached otherwise
		~Nogood_Base_AC ();						///< destructor

		// basic functions
		virtual bool Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds, int var) = 0;		///< achieves, from the variable var (if any), the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
		void Add_Nogood (pair <int,int> * ng,int size);										///< adds the nogood ng involving size variables into the nogood base
		void Add_Nogood (pair <int,int> * ng,int size, Assignment & A);		///< adds the nogood ng involving size variables into the nogood base and selects the watch literals depending on the assignment A
		void Clear ();																										///< clears the nogood base
};
#endif

