#include "not_equal_sum_arithmetic_global_constraint.h"


//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a new constraint which involves the variable in var and which checks whether the sum of the values of the variables in var is not equal to the constant cst
Not_Equal_Sum_Arithmetic_Global_Constraint::Not_Equal_Sum_Arithmetic_Global_Constraint (vector<Variable *> & var, int cst): 
          Sum_Arithmetic_Global_Constraint (var,cst){}


//-----------------
// basic functions
//-----------------


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Not_Equal_Sum_Arithmetic_Global_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
	unsigned int x = Get_Position (var);
	for (unsigned int i = 0; i < arity; i++)
		if ((i != x) && (scope_var [i]->Get_Domain()->Get_Size() != 1))   return false;

	int sum = 0;
	
	for (unsigned int i = 0; i < arity; i++)
		if (i != x)  sum += scope_var [i]->Get_Domain()->Get_Remaining_Real_Value (0);
    
	Domain * dx = scope_var [x]->Get_Domain();
	int dx_size = dx->Get_Size();
	
	int i = 0;
	bool modif = false;
	int v;
	
	do {
		v = dx->Get_Remaining_Real_Value (i);
		
		if (sum+v == constant) {
			v = dx->Get_Remaining_Value (i);
			ds->Add_Element (dx,v);
			dx->Filter_Value (v);
			dx_size--;
			
			modif = true;
		}
		else i++;
	}
	while ((i < dx_size) && (! modif));
	
	if (dx->Get_Size () == 0)  weight++;
	
  return modif;
}
