#include "positive_nary_extension_constraint.h"
#include "avl_iterator.h"


// constructs a new constraint with number n which involves the variable in var whose all the tuples are forbidden
Positive_Nary_Extension_Constraint::Positive_Nary_Extension_Constraint (vector<Variable *> & var): Nary_Extension_Constraint (var) {}


//-----------------
// basic functions
//-----------------


// merges the current constraint with the constraint c
void Positive_Nary_Extension_Constraint::Merge (Positive_Nary_Extension_Constraint * c) {
	if (this != c) {
		int pos[arity];
		for (unsigned int i = 0; i < arity; i++)
			pos[i] = c->Get_Position (scope_var[i]->Get_Num());

		AVL * old_relation = relation;
		relation = new AVL (scope_var);
		int size = scope_var[0]->Get_Domain()->Get_Initial_Size();
		int t [arity];
		
		for (int i = 0; i < size; i++) {
			for (AVL::iterator iter = old_relation->begin(0,i); iter != old_relation->end(0); iter++) {
				for (unsigned int j = 0; j < arity; j++)
					t[pos[j]] = (*iter)[j];
				
				if (c->relation->Is_Present (t))	relation->Add_Element (*iter);
			}
		}
		delete old_relation;
	}
}


// allows the tuple t
void Positive_Nary_Extension_Constraint::Allow_Tuple (int * t) {
	relation->Add_Element (t);
}


// returns true if the tuple is valid w.r.t. the domains of the variables in var
bool Is_Valid (Light_Set<Variable *> var, int * values) {
	bool ok = true;
	int i = 0;
	for (Light_Set<Variable *>::iterator iter = var.begin(); (ok) && (iter != var.end()); iter++) {
		if (values[i] == -1)  ok = false;
		else if ((*iter)->Get_Domain()->Is_Present (values[i]))  i++;
         else ok = false;
	}
	return ok;
}


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Positive_Nary_Extension_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  int x = Get_Position (var);
  
	Domain * dx = scope_var [x]->Get_Domain();
	int dx_size = dx->Get_Size();
	
	int i = 0;
	int v;
		
  int * ts;
  bool has_no_support;
  bool record_support;
  bool modif = false;    // true if a deletion is achieved, false otherwise
  AVL::iterator iter;
	
	do {    
		v = dx->Get_Remaining_Value (i);
		
		ts = ls->Get_Support (num,x,v);
		
		if (ts == 0) {
			iter = relation->begin (x,v);
			
			has_no_support = true;
			record_support = true;
		}
		else if (Is_Valid (scope_var,ts)) {
           has_no_support = false;
           record_support = false;
         }
				 else {
             if (ls->Continue_From_Scratch())  iter = relation->begin (x,v);
             else iter = relation->begin (x,ts);
             has_no_support = true;
             record_support = true;
           }
			
		while ((has_no_support) && (iter != relation->end(x))) {
			if (Is_Valid (scope_var,*iter)) 	has_no_support = false;
			else iter++;			
		}
		
		if (has_no_support) {
			ds->Add_Element (dx,v);
			dx->Filter_Value (v);
			dx_size--;

			modif = true;
		}
		else {
				if (record_support) {
					ds->Add_Support (num,x,v,arity,ts);
					ls->Change_Support (num,x,v,*iter,arity);
				}
				i++;
			}
  }
  while (i < dx_size);
  
	if (dx->Get_Size () == 0)  weight++;
    
  return modif;
}
