#include "predicate_constraint.h"


//-----------------------------
// constructors and destructor
//-----------------------------


// constructs a new constraint with number n which involves the variable in var and whose relation is given by the expression e
Predicate_Constraint::Predicate_Constraint (vector<Variable *> & var, Expression & e): Constraint (var), t (var.size()), relation (e) {}


//-----------------
// basic functions
//-----------------


bool is_present (Expression & e, Expression & e2) {
	operator_type op = e.Get_Operator();

	if ((e.Get_Type () == OP) && (op == AND)) {
		if ((e2 == e.Get_Subexpression(0)) || (e2 == e.Get_Subexpression(1)))	return true;
		else {
				int i = 0;
				int arity = e.Get_Arity ();
				while ((i < arity) && (! is_present (e.Get_Subexpression(i),e2)))  i++;
        
				return i < arity;
			}
	}
	else return e == e2;
}

// merges the current constraint with the constraint c
void Predicate_Constraint::Merge (Predicate_Constraint * c) {
	arity = scope.size();	
	vector<int> effective_parameters (arity);
	vector<node_type> effective_parameter_type (arity);
	
	for (unsigned int i = 0; i < arity; i++) {
		effective_parameters[i] = Get_Position (c->scope_var[i]->Get_Num());
		effective_parameter_type [i] = VAR;
	}
	
	Expression subexpr[2];
	subexpr[0] = relation;
	subexpr[1] = c->relation;
	subexpr[1].Change_Variable (effective_parameters,effective_parameter_type);
	
	if (! is_present (subexpr[0],subexpr[1]))  relation = Expression (AND,subexpr,2);
}


// returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
bool Predicate_Constraint::Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds) {
  int x = Get_Position (var);
  
	Domain * dx = scope_var [x]->Get_Domain();
	int dx_size = dx->Get_Size();
	
	int i = 0;
	int v;
	
  int * ts;
  bool has_no_support;
  bool record_support;
  bool modif = false;    // true if a deletion is achieved, false otherwise
	bool exist_more_tuples;
	
	do {
		v = dx->Get_Remaining_Value (i);
		
		ts = ls->Get_Support (num,x,v);
		
		if (ts == 0) {
			t.Assign (x,v,dx->Get_Real_Value(v));
			t.Complete (scope_var,x);
			has_no_support = true;
			record_support = true;
			exist_more_tuples = true;
		}
		else {
				for (unsigned int j = 0; j < arity; j++)  t.Assign (j,ts[j],scope_var[j]->Get_Domain()->Get_Real_Value(ts[j]));

				if (t.Is_Valid (scope_var)) {
					has_no_support = false;
					record_support = false;
					exist_more_tuples = true;
				}
				else {
						has_no_support = true;
						record_support = true;
						
						if (ls->Continue_From_Scratch()) {
							t.Assign (x,v,dx->Get_Real_Value(v));
							t.Complete (scope_var,x);
							exist_more_tuples = true;							
						}
						else exist_more_tuples = t.Next_Compatible_Valid (scope_var,x);
					}
			}
				
		while ((has_no_support) && (exist_more_tuples))
			if (relation.Evaluate(t.Get_Real_Values()) != 0)	has_no_support = false;
			else exist_more_tuples = t.Next_Valid (scope_var,x);
		
		if (has_no_support) {
			ds->Add_Element (dx,v);
			dx->Filter_Value (v);
			dx_size--;
			
			modif = true;
		}
		else  {
				if (record_support) {
					ds->Add_Support (num,x,v,arity,ts);
					ls->Change_Support (num,x,v,t.Get_Values(),arity);
				}
				i++;
			}
  }
  while (i < dx_size);
  
	if (dx->Get_Size () == 0)  weight++;
  
  return modif;
}
