/// \file 
/// \brief Definitions related to the Predicate_Constraint class

#ifndef PREDICATE_CONSTRAINT_H
#define PREDICATE_CONSTRAINT_H


#include "constraint.h"
#include "expression.h"
#include "real_tuple.h"


class Predicate_Constraint: public Constraint {      /// This class implements the predicate constraint
	private:
		Real_Tuple t;						///< the tuple used in Revise (the declaration of this member is only motivated by runtime reason)
	
	protected:
		Expression relation;		///< the relation related to the constraint
		
	public:
		// constructors and destructor
		Predicate_Constraint (vector<Variable *> & var, Expression & e); 	///< constructs a new constraint which involves the variable in var and whose relation is given by the expression e

		// basic functions
		void Merge (Predicate_Constraint * c);     	///< merges the current constraint with the constraint c
		Expression & Get_Relation ();						 	 	///< returns the relation
		virtual bool Revise (CSP * pb, int var, Multiple_Residue_Support * ls, Deletion_Stack * ds);	 ///< returns true if the application of arc-consistency on the constraint w.r.t. the variable var deletes a value in the domain of var, false otherwise
};


//-----------------------------
// inline function definitions
//-----------------------------


// returns the relation
inline Expression & Predicate_Constraint::Get_Relation () {
	return relation;	
}

#endif
