/// \file 
/// \brief Definitions related to the Restart_Policy class

#ifndef RESTART_POLICY_H
#define RESTART_POLICY_H

using namespace std;

class Restart_Policy  {   /// This class implements the geometric restart policy based on the number of performed backtracks
  protected:
    double scaling_factor;		///< the scaling factor used to increase the number of allowed backtracks after each restart (must be greater than or equal to 1)
		unsigned long limit;			///< the number of allowed backtracks before the next restart
		unsigned long backtrack_number_since_last_restart;		///< the number of backtrack since the last restart
  
	public:
    // constructor
    Restart_Policy (unsigned long lim, double sc_factor);		///< creates an new policy with limit lim and scaling factor sc_factor
		
		// basic functions
		bool Restart ();	 ///< returns true if the solver has to restart according to the given policy and reset all information required by the policy, false otherwise
		void Reset ();	 	 ///< resets the policy parameter
};


//-----------------------------
// inline function definitions
//-----------------------------

// resets the policy parameter (i.e. here the number of backtrack since the last restart)
inline void Restart_Policy::Reset () {
	backtrack_number_since_last_restart = 0;
}

#endif
