/// \file 
/// \brief Definitions related to the Value_Heuristic class

#ifndef VALUE_HEURISTIC_H
#define VALUE_HEURISTIC_H

using namespace std;

#include "csp.h"

class Value_Heuristic {   /// This class implements the lexicographical value heuristic
	public:	
		// basic function
		int Next_Value (CSP * pb, int var, int previous_val = -1);	 ///< returns the value of the variable var of the CSP pb which follows the value previous_val in the lexicographical order
};


//-----------------------------
// inline function definitions
//-----------------------------


// returns the value of the variable var of the CSP pb which follows the value previous_val in the lexicographical order
inline int Value_Heuristic::Next_Value (CSP * pb, int var, int previous_val) {
	Domain * d = pb->Get_Variable(var)->Get_Domain();
	
	int i = previous_val + 1;
	while (! d->Is_Present (i))  i++;
		
	return i;
}
#endif
