#include "variable_heuristic.h"

//-------------
// constructor
//-------------


// constructs a variable heuristic which can manege at most n candidates
Variable_Heuristic::Variable_Heuristic (unsigned int n): candidates (n) {}


//-----------------
// basic functions
//-----------------


// duplicates the variable heuristic

Variable_Heuristic * Variable_Heuristic::Duplicate () {
	return new Variable_Heuristic (candidates.get_capacity());	
}


// returns the variable of the CSP pb which has been selected among the variables unassigned in A according to the min (domain size / weighted degree) heuristic
Variable * Variable_Heuristic::Next_Variable (CSP * pb, Assignment & A) {
	Variable * x = 0;
	double max = -1.0;
	double ratio;
	Light_Set<Variable *>::iterator iter2;
	Multi_Hypergraph * h = pb->Get_Structure ();
	Light_List<Edge *>::iterator iter3;
	Constraint * c;
	int num;
	
	if (candidates.size() == 1) {
		iter2 = candidates.begin();
		x = *iter2;
	}
	else {
			for (Light_Set<Variable * >::iterator iter = candidates.begin(); iter != candidates.end(); iter++)
				if (! A.Is_Assigned ((*iter)->Get_Num())) {
					num = (*iter)->Get_Num();
					ratio = 0.0;
					for (iter3 = h->Begin_Edge_List (num); iter3 != h->End_Edge_List (num); iter3++) {
						c = pb->Get_Constraint ((*iter3)->Get_Num());
						if (c->Get_Unassigned_Variable_Number() > 1)	ratio += c->Get_Weight();
					}
					
					ratio /= (*iter)->Get_Domain()->Get_Size();
					
					if ((ratio > max) || ((ratio == max) && ((*iter)->Get_Num() < x->Get_Num()))) {
						max = ratio;
						x = *iter;
						iter2 = iter;
					}
				}
		}
	
	return x;
}
