/// \file 
/// \brief Definitions related to the Variable_Heuristic class

#ifndef VARIABLE_HEURISTIC_H
#define VARIABLE_HEURISTIC_H

#include <light_set.h>
#include "csp.h"
#include "assignment.h"


class Variable_Heuristic {        /// This class implements the dom/wdeg variable heuristic
	protected:
		Light_Set<Variable *> candidates;		///< the set of variable candidates
	
	public:	
		// constructor
		Variable_Heuristic (unsigned int n);		///< constructs a variable heuristic which can manege at most n candidates
		
		// basic functions
		void Add_Candidate (Variable * x);	 		///< adds the variable x to the candidate set
		void Remove_All_Candidates ();				  ///< removes all the candidates from the candidate set

		Variable_Heuristic * Duplicate ();	  ///< duplicates the variable heuristic
		Variable * Next_Variable (CSP * pb, Assignment & A);	///< returns the variable of the CSP pb which has been selected among the variables unassigned in A according to the min (domain size / weighted degree) heuristic
};


//-----------------------------
// inline function definitions
//-----------------------------


// adds the variable x to the candidate set
inline void Variable_Heuristic::Add_Candidate (Variable * x) {
	candidates.add (x);
}


// removes all the candidates from the candidate set
inline void Variable_Heuristic::Remove_All_Candidates () {
	candidates.clear ();
}
#endif
