#include "variable_nogood_base_ac.h"
#include "deletion_stack.h"


//----------------------------
// constructor and destructor
//----------------------------

// constructs an AC algorithm using support structure ls, the constraint selection policy cs and the AC data structure acds which will be achieved in one-pass if one_pass is true, until the fix-point is reached otherwise
Variable_Nogood_Base_AC::Variable_Nogood_Base_AC (CSP * ref_pb, Multiple_Residue_Support * ls, Constraint_Selection * cs, Filtering_Data_Structure<int> * acds, 
                                                  bool one_pass): Nogood_Base_AC (ref_pb,ls,cs,one_pass) {
	structure = acds;
}


// destructor
Variable_Nogood_Base_AC::~Variable_Nogood_Base_AC () {
	delete structure;
}


//-----------------
// basic functions
//-----------------


// achieves, from the variable var (if any), the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise
bool Variable_Nogood_Base_AC::Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds, int var) {
	is_selected.resize (pb->Get_N());
  
  if ((var == -1) && (A.Get_Size () == 0)) {
		for (unsigned int i = 0; i < pb->Get_N(); i++) {
			structure->Insert (i);
			is_selected[i] = true;
		}
		var = -1;
	}
	else {
			for (unsigned int i = 0; i < pb->Get_N(); i++)
				is_selected[i] = false;

			if (var == -1)  var = A.Last_Assigned();
			
			structure->Insert (var);
			is_selected[var] = true;
		}

	int k;
	Multi_Hypergraph * h = pb->Get_Structure();
	Light_List<Edge*>::iterator iter;
	set<vertex>::iterator iter2;
	Constraint * c;
	Domain * d;

	while (! structure->Is_Empty ()) {
  	k = structure->Current ();
  	structure->Remove();

    is_selected[k] = onepass;
    
    // we check the nogood base if necessary
    d = pb->Get_Variable(k)->Get_Domain();
    if (d->Get_Size() == 1) {
			if (! Check_Watches (k,d->Get_Remaining_Value(0),ds)) {
				structure->Clear ();
				return false;
			}
		}

		for (iter = h->Begin_Edge_List(k); iter != h->End_Edge_List(k); iter++) {
			c = pb->Get_Constraint((*iter)->Get_Num());
			
			if (selection->Is_Eligible (c,var)) {
				if (c->Has_Specific_Propagator()) {
					bool * modified;
					c->Propagate (pb,A,supports,ds,modified);
					
					int num;
					for (unsigned int i = 0; i < c->Get_Arity(); i++)
						if (modified[i]) {
							if (c->Get_Scope_Variable(i)->Get_Domain()->Get_Size() == 0) {
								structure->Clear ();
								return false;
							}
							
							if (! is_selected[num = c->Get_Scope_Variable(i)->Get_Num()]) { 
								is_selected[num] = true;
								
								structure->Insert (num);
							}
						}
				}
				else {
						for (iter2 = (*iter)->Begin(); iter2 != (*iter)->End(); iter2++) {
							if ((signed) *iter2 != k) {
								if ((! A.Is_Assigned (*iter2)) && (c->Revise (pb,*iter2,supports,ds))) {
									if (pb->Get_Variable(*iter2)->Get_Domain()->Get_Size() == 0) {
										structure->Clear ();
										return false;
									}

									if (! is_selected[*iter2]) { 
										is_selected[*iter2] = true;
										
										structure->Insert (*iter2);
									}
								}
							}
						}
					}
			}
    }
  }
  return true;
}


// looks for another watched pair (variable,value) for each nogood involving (var,val) and infers value deletion if possible
bool Variable_Nogood_Base_AC::Check_Watches (int var,int val,Deletion_Stack * ds) {
	int i, position;
	Domain * d;
	Light_List<element*>::iterator iter2;

	for (Light_List<element*>::iterator iter = nogood_base[var][val].begin (); iter != nogood_base[var][val].end (); ) {
		// we look for another watch 
		position = -1;
		i = 0;
		
		while ((i < (*iter)->size) && (position == -1)) {
			if (((*iter)->watch1 != i) && ((*iter)->watch2 != i)) {
				d = pb->Get_Variable((*iter)->nogood[i].first)->Get_Domain();
				if ((! d->Is_Present((*iter)->nogood[i].second)) || (d->Get_Size() > 1))
					position = i;
			}
			i++;
		}
		if (position == -1) {
			if ((*iter)->nogood[(*iter)->watch1].first == var)  position = (*iter)->watch2;
			else position = (*iter)->watch1;
			
			d = pb->Get_Variable((*iter)->nogood[position].first)->Get_Domain();
			if (d->Is_Present((*iter)->nogood[position].second)) {
				d->Filter_Value((*iter)->nogood[position].second);
				ds->Add_Element (d,(*iter)->nogood[position].second);
				
				if (d->Get_Size() == 0)  return false;
				else 
					if (! is_selected[(*iter)->nogood[position].first]) {
						structure->Insert((*iter)->nogood[position].first);
						is_selected [(*iter)->nogood[position].first] = true;
					}
			}
			iter++;
		}
		else {
				iter2 = iter;
				nogood_base[var][val].skip (iter);
				if ((*iter2)->nogood[(*iter2)->watch1].first == var)  (*iter2)->watch1 = position;
				else (*iter2)->watch2 = position;
				
				nogood_base[(*iter2)->nogood[position].first][(*iter2)->nogood[position].second].push_front (iter2);
			}
	}

	return true;
}
