/// \file 
/// \brief Definitions related to the Variable_Nogood_Base_AC class

#ifndef VARIABLE_NOGOOD_BASE_AC_H
#define VARIABLE_NOGOOD_BASE_AC_H

#include "nogood_base_ac.h"
#include "filtering_data_structure.h"


class Variable_Nogood_Base_AC: public Nogood_Base_AC  {    /// This class implements variable-oriented AC algorithm with nogood base
	protected:
		Filtering_Data_Structure<int> * structure; 			///< the data structure representing the set of the constraints for which the consistency must be checked
		vector<bool> is_selected;												///< true if the variable x is in the structure, false otherwise

	public:
		// constructor and destructor
		Variable_Nogood_Base_AC (CSP * ref_pb, Multiple_Residue_Support * ls, Constraint_Selection * cs, Filtering_Data_Structure<int> * acds, bool one_pass = false);	///< constructs an AC algorithm using support structure ls, the constraint selection policy cs and the AC data structure acds which will be achieved in one-pass if one_pass is true, until the fix-point is reached otherwise
		~Variable_Nogood_Base_AC ();						///< destructor

		// basic functions
		virtual bool Achieve (CSP * pb, Assignment & A, Deletion_Stack * ds, int var);	///< achieves, from the variable var (if any), the current AC algorithm on the CSP pb under the assignment A and returns true if the problem is consistent w.r.t. this algorithm, false otherwise (the deletion of values are recorded in the stack ds)
	
	protected:
		bool Check_Watches (int var,int val,Deletion_Stack * ds);		///< looks for another watched pair (variable,value) for each nogood involving (var,val) and infers value deletion if possible
};
#endif

