#include "connected_component.h"
#include <typeinfo>
#include "light_graph.h"

//----------------------------
// constructor and destructor
//----------------------------

// constructs a related to the graphical structure gs
Connected_Component::Connected_Component (Graphical_Structure * gs) {
	if (typeid(*gs) == typeid (Light_Graph))  g = dynamic_cast<Light_Graph*>(gs);
	else g = gs->Get_Primal_Light_Graph();
}


//-----------------				
// basic functions
//-----------------


// returns the list of connected components of the related graphical substructure induced by the vertex set vs - except_vs
Light_List<pair<int,vertex *> > Connected_Component::Get_Connected_Components (pair<int,vertex *> & vs, pair<int,vertex *> & except_vs) {
	unsigned int n = g->N();
  bool * visited = new bool [n];;            	// true if the vertex x has been visited, false otherwise
  
  int nb_remaining_vertices = vs.first; 	// the number of remaining vertices
  vertex x;
  Light_List<pair<int,vertex *> > cc_list;		// the list of built connected components

  // initializations
  for (x =0; x < n; x++)	visited[x] = true;
  
  for (int i = 0; i < vs.first; i++)	visited[vs.second[i]] = false;

  for (int i = 0; i < except_vs.first; i++)
		if (! visited[except_vs.second[i]])	{
			visited[except_vs.second[i]] = true;
			nb_remaining_vertices--;
		}
		
  Light_Stack<vertex> s (nb_remaining_vertices);
  vertex v;

  // the computation is achieved by a depth-first search of the graph
  x = 0;
  vertex * t = new vertex [n];	// the vertices of the connected component of x

  int nb = 0;
  while (nb_remaining_vertices > 0) {
    if (! visited[x]) {
			nb++;
			// traversal of the connected component of the vertex x
			int card = 1;		// the cardinal of the vertex set corresponding to the connected component of x
			
			visited[x] = true;
			t[0] = x;
			nb_remaining_vertices--;

			s.push(x);

			while (! s.is_empty()) {
				v = s.top();
				s.pop();

				for (Light_List<vertex>::iterator iter=g->Begin_Neighbor (v); iter != g->End_Neighbor (v); iter++)
					if (! visited[*iter])	{
						s.push (*iter);
						visited[*iter] = true;
						t[card] = *iter;
						card++;
						nb_remaining_vertices--;
					}
			}
			
			vertex * t2 = new vertex [card];
			for (int i = 0; i < card; i++)	t2[i] = t[i];
      
			cc_list.push_back (make_pair(card,t2));
		}
		x++;
  }

	delete [] t;
	delete [] visited;
  return cc_list;
}


// puts in the stack S the list of both connected components of the related graphical substructure induced by the vertex set vs - except_vs, and their related neighborhood in except_vs
void Connected_Component::Get_Connected_Components_With_Neighborhood (pair<int,vertex *> & vs, Cluster_Node * except_vs, Light_Stack<tuple<pair<int,vertex *>,
                                                                      Cluster_Node *,pair<int,vertex *>>> & S) {
	unsigned int n = g->N();
  int*  visited = new int [n];  	// 0 if the vertex has not been visited yet, 1 if if the vertex is in except_vs and has not been reached yet from the current vertex x, 2 otherwise
  int nb_remaining_vertices = vs.first; 	// the number of remaining vertices

  // initializations
  for (vertex x =0; x < n; x++) 	visited[x] = 2;
  
  for (int i = 0; i < vs.first; i++)	visited[vs.second[i]] = 0;

  for (Light_Vector<vertex>::iterator iter_v = except_vs->begin(); iter_v != except_vs->end(); iter_v++) {
		if (visited[*iter_v] == 0)	nb_remaining_vertices--;
		visited[*iter_v] = 1;
	}
		
  Light_Stack<vertex> s (nb_remaining_vertices);
  vertex v;

  // the computation is achieved by a depth-first search of the graph
  vertex x = 0;
  vertex * t = new vertex [nb_remaining_vertices];	// the vertices of the connected component of x
  
  vertex * V = new vertex [except_vs->Size()];	// the neighborhood 

  while (nb_remaining_vertices > 0)  {
    if (visited[x] == 0) {
			// traversal of the connected component of the vertex x
			int card = 1;			// the cardinal of the vertex set corresponding to the connected component of x
			int card_V = 0;		// the cardinal of the neighborhood
			
			visited[x] = 2;
			t[0] = x;
			nb_remaining_vertices--;

			s.push(x);

			while (! s.is_empty()) {
				v = s.top();
				s.pop();

				for (Light_List<vertex>::iterator iter=g->Begin_Neighbor (v); iter != g->End_Neighbor (v); iter++)
					if (visited[*iter] == 0)	{
						s.push (*iter);
						visited[*iter] = 2;
						t[card] = *iter;
						card++;
						nb_remaining_vertices--;
					}
					else if (visited[*iter] == 1)	{
							// we found a vertex in the neighborhood
							V[card_V] = *iter;
							card_V++;
							visited[*iter] = 2;
						}
			}
			
			vertex * t2 = new vertex [card];
			for (int i = 0; i < card; i++)  t2[i] = t[i];

			vertex * t3 = new vertex [card_V];
			for (int i = 0; i < card_V; i++)  t3[i] = V[i];
			
			S.push (make_tuple(make_pair(card,t2), except_vs, make_pair(card_V,t3)));

			if (nb_remaining_vertices > 0)
				for (Light_Vector<vertex>::iterator iter_v = except_vs->begin(); iter_v != except_vs->end(); iter_v++)
					visited[*iter_v] = 1;
		}
		x++;
  }

	delete [] visited;
	delete [] t;
	delete [] V;
}


// puts in the stack S the list of both connected components of the related graphical substructure induced from the vertex set initial_vs, and their related neighborhood in initial_vs
void Connected_Component::Get_Connected_Components_From_Set (Cluster_Node * initial_vs, Light_Stack<tuple<pair<int,vertex *>,Cluster_Node *, 
                                                             pair<int,vertex *>>> & S) {
	unsigned int n = g->N();
  int * visited = new int [n];  	// 0 if the vertex has not been visited yet, 1 if if the vertex is in initial_vs and has not been reached yet from the current vertex x, 2 otherwise

  // initializations
  for (vertex x =0; x < n; x++) visited[x] = 0;

  for (Light_Vector<vertex>::iterator iter_v = initial_vs->begin(); iter_v != initial_vs->end(); iter_v++)	visited[*iter_v] = 1;
 
  Light_Stack<vertex> s (n-initial_vs->Size());
  vertex v;

  // the computation is achieved by a depth-first search of the graph
  vertex x = 0;
  vertex * t = new vertex [n-initial_vs->Size()];	// the vertices of the connected component of x
  vertex * V = new vertex [initial_vs->Size()];	// the neighborhood 

	for (x = 0; x < n; x++)
    if (visited[x] == 0) {
			// we check whether x is a neighbor of a vertex in initial_vs
			Light_Vector<vertex>::iterator iter_v = initial_vs->begin(); 

			while ((iter_v != initial_vs->end()) && (! g->Is_Edge(x,*iter_v)))	iter_v++;
			
			if (iter_v != initial_vs->end())  {			
				// traversal of the connected component of the vertex x
				int card = 1;		// the cardinal of the vertex set corresponding to the connected component of x
				int card_V = 0;	// the cardinal of the neighborhood
					
				visited[x] = 2;
				t[0] = x;

				s.push(x);

				while (! s.is_empty()) {
					v = s.top();
					s.pop();

					for (Light_List<vertex>::iterator iter=g->Begin_Neighbor (v); iter != g->End_Neighbor (v); iter++)
						if (visited[*iter] == 0) {
							// we add the new vertex in t 
							s.push (*iter);
							visited[*iter] = 2;
							t[card] = *iter;
							card++;
						}
						else if (visited[*iter] == 1)	{
								// we found a vertex in the neighborhood
								V[card_V] = *iter;
								card_V++;
								visited[*iter] = 2;
							}
				}
				
				vertex * t2 = new vertex [card];
				for (int i = 0; i < card; i++)	t2[i] = t[i];
					
				vertex * t3 = new vertex [card_V];
				for (int i = 0; i < card_V; i++)	t3[i] = V[i];

				S.push (make_tuple(make_pair(card,t2), initial_vs, make_pair(card_V,t3)));

				for (Light_Vector<vertex>::iterator iter_v = initial_vs->begin(); iter_v != initial_vs->end(); iter_v++)	visited[*iter_v] = 1;
			}
		}
	
	delete [] visited;
	delete [] t;
	delete [] V;
}
