/// \file 
/// \brief Definitions related to the Edge class

#ifndef EDGE_H
#define EDGE_H

#include <set>
#include "graphical_structure.h"

class Edge {       /// This class allows to represent the edges of hypergraphs
	private :
		int num;        				///< number of the edge
		set<vertex> scope;			///< the vertices which belong to the edge
		
	public :
		// constructors and destructor
		Edge (int num_edge, vertex elements[], int arity);  ///< constructs an edge whose number is num_edge and which involves the vertices from the array elements
		Edge (Edge & e);                    		            ///< constructs an edge by copying the edge e

		// operator
		Edge & operator= (const Edge & e); 									///< operator =
		bool operator== (const Edge & e); 									///< operator ==
		

		// iterators
		set<vertex>::iterator Begin();										///< returns an iterator referring to the first vertex of the vertex set
		set<vertex>::iterator End();										  ///< returns an iterator referring to the position after the vertex of the vertex set

		// basic operations or functions on graphs
		int Get_Num ();									///< returns the number of the edge
		void Set_Num (int n);						///< sets the number of the edge to the value n
		int Get_Arity();								///< returns the arity
		vertex Get_Element (int n);  		///< returns the nth vertex of the scope
		void Add_Vertex (vertex x);			///< adds the vertex x to the edge
};


//-----------------------------
// inline function definitions
//-----------------------------


//-----------
// iterators
//-----------

// returns an iterator referring to the first vertex of the vertex set
inline set<vertex>::iterator Edge::Begin() {
	return scope.begin();	
}


// returns an iterator referring to the position after the vertex of the vertex set
inline set<vertex>::iterator Edge::End() {
	return scope.end();
}


//-----------------------------------------
// basic operations or functions on graphs
//-----------------------------------------


// returns the number of the edge
inline int Edge::Get_Num () {
	return num;
}


// sets the number of the edge to the value n
inline void Edge::Set_Num (int n) {
	num = n;
}


// returns the arity
inline int Edge::Get_Arity() {
	return scope.size();
}


// adds the vertex x to the edge
inline void Edge::Add_Vertex (vertex x) {
	scope.insert (x);
}

#endif

