#include "light_graph.h"

//-----------------------------
// constructors and destructor
//-----------------------------

// constructs an empty graph with 0 vertex and 0 edge
Light_Graph::Light_Graph (): Graphical_Structure() {
}


// constructs an graph by copying the graph g
Light_Graph::Light_Graph (Light_Graph & g) {
	// initialization of the graph with g.n vertices and 0 edge
	n = g.n;
	m = 0;
	plg = 0;

	adjacency_list.expand (n);
	degree.expand (n,0);
	  
	// copy of the edges of the graph g
	
	for (vertex x = 0; x < n; x++)
    for (Light_List<vertex>::iterator iter = g.adjacency_list[x].begin(); iter != g.adjacency_list[x].end(); iter++)
      if (x < *iter)  Add_Edge (x,*iter);
}



//----------
// operator
//----------

// operator =
Light_Graph & Light_Graph::operator= (Light_Graph & g) {
  if (this != &g) {
		// initialization of the graph with g.n vertices and 0 edge
		n = g.n;
		m = 0;
		plg = 0;

		adjacency_list.expand(n);
		degree.expand (n,0);

			
		// copy of the edges of the graph g
		
		for (vertex x = 0; x < n; x++)
			for (Light_List<vertex>::iterator iter = g.adjacency_list[x].begin(); iter != g.adjacency_list[x].end(); iter++)
				if (x < *iter)	Add_Edge (x,*iter);
	}
	
	return *this;
}



//----------------------------
// basic operations on graphs
//----------------------------


// initializes the graph with n vertices and 0 edge
void Light_Graph::Init (unsigned int nb_vertices) {
	n = nb_vertices;
	m = 0;
	adjacency_list.expand (n);
	degree.expand (n,0);
}


// adds a vertex to the graph
void Light_Graph::Add_Vertex () {
	n++;
	adjacency_list.expand (1);
	degree.expand (1,0);
}


// returns true if {x,y} is an edge of the graph, false otherwise
bool Light_Graph::Is_Edge (vertex x, vertex y) {
	Light_List<vertex>::iterator iter;
	for (iter = adjacency_list[x].begin(); (iter != adjacency_list[x].end()) && (*iter != y); iter++);
	
  return iter != adjacency_list[x].end();
}


// adds the vertices of elements as a new edge of the graph if the value of arity is 2, otherwise adds these vertices as a new clique of the graph
void Light_Graph::Add_Edge (vertex elements[], int arity) {
	for (int i=0; i < arity; i++)
		for (int j=i+1; j < arity; j++)
			Add_Edge (elements[i],elements[j]);
}


// adds the edge {x,y} to the graph
void Light_Graph::Add_Edge (vertex x, vertex y) {
	if (! Is_Edge (x,y))	{
		adjacency_list[x].push_front (y);
		adjacency_list[y].push_front (x);
		degree[x]++;
		degree[y]++;
		m++;
		plg = 0;
	}
}
