/// \file 
/// \brief Definitions related to the Light_Graph class

#ifndef LIGHT_GRAPH_H
#define LIGHT_GRAPH_H

#include "graphical_structure.h"
#include <light_list.h>

class Light_Graph: public Graphical_Structure  {         /// This class allows to represent graphs with adjacency list
  protected : 
  	Light_Vector<Light_List<vertex> > adjacency_list;   ///< the adjacency list of the graph
		
	public :
	  // constructors and destructor
		Light_Graph ();       					 							///< constructs an empty graph with 0 vertex and 0 edge
		Light_Graph (Light_Graph & g);         				///< constructs an graph by copying the graph g

		// operator
		Light_Graph & operator= (Light_Graph & g); 		///< operator =

    // iterators
		Light_List<vertex>::iterator Begin_Neighbor (vertex x); ///< returns an iterator referring to the first neighbor of the vertex x
		Light_List<vertex>::iterator End_Neighbor (vertex x);   ///< returns an iterator referring to the position after the last neighbor of the vertex x
    
    // basic operations or functions on graphs
		virtual void Init (unsigned int nb_vertices); 					///< initializes the graph with n vertices and 0 edge
		virtual void Add_Vertex ();   													///< adds a vertex to the graph
		virtual bool Is_Edge (vertex x, vertex y);   						///< returns true if {x,y} is an edge of the graph, false otherwise
		virtual void Add_Edge (vertex elements[], int arity);  	///< adds the vertices of elements as a new edge of the graph if the value of arity is 2, otherwise adds these vertices as a new clique of the graph
		virtual void Add_Edge (vertex x, vertex y);   					///< adds the edge {x,y} to the graph


    // miscellaneous functions
    virtual Light_Graph * Get_Primal_Light_Graph ();	 ///< returns the primal graph related to the graphical structures as a pointer on a Light_Graph object (i.e. a copy of the graph itself)
};


//-----------------------------
// inline function definitions
//-----------------------------

//-----------
// iterators
//-----------


// returns an iterator referring to the first neighbor of the vertex x
inline Light_List<vertex>::iterator Light_Graph::Begin_Neighbor (vertex x) {
	return adjacency_list[x].begin();
}


// returns an iterator referring to the position after the last neighbor of the vertex x
inline Light_List<vertex>::iterator Light_Graph::End_Neighbor (vertex x) {
	return adjacency_list[x].end();
}


// returns the primal graph related to the graphical structures as a pointer on the Light_Graph object itself
inline Light_Graph * Light_Graph::Get_Primal_Light_Graph () {
	return this;
}

#endif
