#include "multi_hypergraph.h"

//-----------------------------
// constructors and destructor
//-----------------------------


// constructs an empty hypergraph with 0 vertex and 0 edge
Multi_Hypergraph::Multi_Hypergraph (): Graphical_Structure () {}


// destructor
Multi_Hypergraph::~Multi_Hypergraph () {
	Light_List<Edge *> to_be_deleted;
	
	for (vertex x = 0; x < n; x++)
    for (Light_List<Edge *>::iterator iter = adjacency_list[x].begin(); iter != adjacency_list[x].end(); iter++)
      if (x == (*iter)->Get_Element(0))  to_be_deleted.push_front(*iter);
  
  for (Light_List<Edge *>::iterator iter = to_be_deleted.begin(); iter != to_be_deleted.end(); iter++) 	delete *iter;
}



//----------------------------------------------------
// basic operations or functions on Multi_Hypergraphs
//----------------------------------------------------


// initializes the hypergraph with n vertices and 0 edge
void Multi_Hypergraph::Init (unsigned int nb_vertices) {
	n = nb_vertices;
	m = 0;
	
	adjacency_list.expand(n);
	degree.expand (n,0);
	delete plg;
	plg = 0;
}


// adds a vertex to the hypergraph
void Multi_Hypergraph::Add_Vertex () {
	n++;
	adjacency_list.expand(1);
	degree.expand (1,0);
	delete plg;
	plg = 0;
}


// adds the vertices of elements as a new edge of the hypergraph
void Multi_Hypergraph::Add_Edge (vertex elements[], int arity) {
	Edge * e = new Edge (m,elements,arity);
	
	for (int i = 0; i < arity; i++) {
		adjacency_list[elements[i]].push_front (e);
		degree[elements[i]]++;
	}
	
	m++;
		
	delete plg;
	plg = 0;
}


//-------------------------
// miscellaneous functions
//-------------------------


// returns the primal graph related to the graphical structures as a pointer on a Light_Graph object
Light_Graph * Multi_Hypergraph::Get_Primal_Light_Graph () {
	if (plg == 0) {		
		// initialization
		plg = new Light_Graph();
		
		plg->Init (N());
		
		// addition of each hyperedge as a clique of the graph
		set<vertex>::iterator iter2,iter3;
		
		for (vertex x = 0; x < n; x++)
			for (Light_List<Edge *>::iterator iter = adjacency_list[x].begin(); iter != adjacency_list[x].end(); iter++)
				if (x == (*iter)->Get_Element(0))
					for (iter2 = (*iter)->Begin(); iter2 != (*iter)->End(); iter2++)
						for (iter3=iter2, iter3++; iter3 != (*iter)->End(); iter3++)
							plg->Add_Edge(*iter2,*iter3);
	}
	
	return plg;
}
