/// \file 
/// \brief Definitions related to the Multi_Hypergraph class

#ifndef MULTI_HYPERGRAPH_H
#define MULTI_HYPERGRAPH_H

#include "graphical_structure.h"
#include "edge.h"
#include "light_graph.h"


class Multi_Hypergraph : public Graphical_Structure  {  /// This class allows to represent hypergraphs with eventually redundant edges
  protected : 
  	Light_Vector<Light_List<Edge *> >  adjacency_list;  ///< the adjacency list of the hypergraph
		
	public :
	  // constructors and destructor
		Multi_Hypergraph ();       					 							   ///< constructs an empty hypergraph with 0 vertex and 0 edge
		~Multi_Hypergraph ();		 					  		  					 ///< destructor

    // iterators
		Light_List<Edge *>::iterator Begin_Edge_List (vertex x); ///< returns an iterator referring to the first edge in the adjacency list of the vertex x
		Light_List<Edge *>::iterator End_Edge_List (vertex x);   ///< returns an iterator referring to the position after the last edge in the adjacency list of the vertex x
    
    // basic operations or functions on hypergraphs
		void Init (unsigned int nb_vertices);  										///< initializes the hypergraph with n vertices and 0 edge
		virtual void Add_Vertex ();   														///< adds a vertex to the hypergraph
		bool Is_Edge (set<vertex> elements);											///< returns true if the edge e is an edge of the hypergraph, false otherwise
		virtual void Add_Edge (vertex elements[], int arity);   	///< adds the vertices of elements as a new edge of the hypergraph


    // miscellaneous functions
		virtual Light_Graph * Get_Primal_Light_Graph ();	 ///< returns the primal graph related to the graphical structures as a pointer on a Light_Graph object
};


//-----------------------------
// inline function definitions
//-----------------------------


//-----------
// iterators
//-----------


// returns an iterator referring to the first edge in the adjacency list of the vertex x
inline Light_List<Edge *>::iterator Multi_Hypergraph::Begin_Edge_List (vertex x) {
	return adjacency_list[x].begin();
}


// returns an iterator referring to the position after the last edge in the adjacency list of the vertex x
inline Light_List<Edge *>::iterator Multi_Hypergraph::End_Edge_List (vertex x) {
	return adjacency_list[x].end();
}

#endif
