/// \file 
/// \brief Definitions related to the Next_Vertices_Heuristic class

#ifndef NEXT_VERTICES_HEURISTIC_H
#define NEXT_VERTICES_HEURISTIC_H

#include "light_graph.h"
#include "cluster_node.h"

class Next_Vertices_Heuristic {    /// This class allows to represent heuristic for choosing the next vertices when computing tree-decompositions by heuristic methods
	protected:
		Light_Graph * g;		///< the original graph
    int k;              ///< the maximal size which is allowed for each separator
	
	public:
		// constructor
		Next_Vertices_Heuristic (Light_Graph * ref_g, int sep_k);		///< constructs a heuristic which chooses vertices in the neighborhood (w.r.t. the graph ref_g) of a set of vertices 
		
		// basic function
		void Choose (pair<int,vertex *> & vs, Cluster_Node * cl, pair<int,vertex *> & sep, int current_w, bool is_available[]);		///< selects some vertices from the vertex set vs (according to the heuristic policy) and adds them to the cluster cl. The selected vertices are removed from vs. According to the heuristic, the selected vertices may have some relation (e.g. neighborhood) with some vertices of the vertex sets aux_vs or sep
};

#endif
