#include "vertex_set.h"
#include "multi_hypergraph.h"


//-----------------------------
// constructors and destructor
//-----------------------------

// constructs an empty vertex set related to gs
Vertex_Set::Vertex_Set (Graphical_Structure * gs) {
	g = gs;
	size = 0;
	nb_edges = 0;
	nb_components = 0;
}


// constructs a vertex set containing the vertices of element_set and related to gs
Vertex_Set::Vertex_Set (Light_Vector<vertex> element_set, Graphical_Structure * gs) {
	g = gs;
	elements = element_set;
	size = elements.get_capacity();

	vertex max = -1;
	for (Light_Vector<vertex>::iterator iter = element_set.begin(); iter != element_set.end(); iter++)
		if (*iter > max) 	max = *iter;
			
	is_element = vector<bool> (max+1,false);
	
	for (Light_Vector<vertex>::iterator iter = elements.begin(); iter != elements.end(); iter++)  is_element[*iter] = true;
	  
	nb_edges = 0;
	nb_components = 0;	
}


// constructs a vertex set by copying the vertex set s
Vertex_Set::Vertex_Set (Vertex_Set & s) {
	g = s.g;
	size = s.size;
	elements = s.elements;
	is_element = s.is_element;
	nb_edges = s.nb_edges;
	nb_components = s.nb_components;
}


// constructs a vertex set by copying the vertex set s
Vertex_Set::Vertex_Set (const Vertex_Set & s) {
	g = s.g;
	size = s.size;
	elements = s.elements;
	is_element = s.is_element;
	nb_edges = s.nb_edges;
	nb_components = s.nb_components;
}


//----------
// operator
//----------


// operator =
Vertex_Set & Vertex_Set::operator= (const Vertex_Set & s) {
	if (this != &s) {
		g = s.g;
		size = s.size;
		elements = s.elements;
		is_element = s.is_element;
		nb_edges = s.nb_edges;
		nb_components = s.nb_components;
	}
	return *this;
}


//------------------------------
// basic function on vertex set
//------------------------------


// adds the vertex v to the current vertex set
void Vertex_Set::Add_Vertex (vertex v) {
	if (v >= is_element.size()) {
		elements.push_back (v);
		is_element.resize (v+1,false);
		is_element[v] = true;
		size++;
	}
	else if (! is_element[v]) {
			elements.push_back (v);
			is_element[v] = true;
			size++;
		}
}


// returns the number of edges included in the vertex set
int Vertex_Set::Get_Nb_Edges () {
	if (nb_edges == 0) 	{
		if (dynamic_cast<Light_Graph*>(g) != 0)	{
			Light_Graph * lg = dynamic_cast<Light_Graph*>(g);
			
			for (Light_Vector<vertex>::iterator iter = elements.begin(); iter != elements.end(); iter++)
				for (Light_List<vertex>::iterator iter2 = lg->Begin_Neighbor(*iter); iter2 != lg->End_Neighbor(*iter); iter2++)
					if (Is_Element(*iter2))  nb_edges++;
			
			nb_edges /= 2;
		}
		else {
				Multi_Hypergraph * mh = dynamic_cast<Multi_Hypergraph*>(g);
				int nb_elements;
				
				for (Light_Vector<vertex>::iterator iter = elements.begin(); iter != elements.end(); iter++)
					for (Light_List<Edge *>::iterator iter2 = mh->Begin_Edge_List(*iter); iter2 != mh->End_Edge_List(*iter); iter2++)	{
						// we look for the first vertex in the edge which also belongs to elements
						set<vertex>::iterator iter3 = (*iter2)->Begin();
						while ((iter3 != (*iter2)->End()) && (! Is_Element(*iter3)))
							iter3++;
						
						if (*iter3 == *iter)	{
							nb_elements = 0;
							while ((iter3 != (*iter2)->End()) && (Is_Element(*iter3))) {
								nb_elements++;
								iter3++;
							}

							if (nb_elements == (*iter2)->Get_Arity())		nb_edges++;
						}
					}
			}
	}
	
	return nb_edges;
}


// adds the vertex set s to the current vertex set
void Vertex_Set::Merge (Vertex_Set * s) {
	if ((this != s) && (s->g == g)) {
		for (Light_Vector<vertex>::iterator iter = s->elements.begin(); iter != s->elements.end(); iter++)
			Add_Vertex (*iter);

		nb_edges = 0;
		nb_components = 0;
	}
}
