//-----------------------------
// constructors and destructor
//-----------------------------

// constructs an empty list
template <class T> Light_List<T>::Light_List () {
	first = 0;
	last = 0;
}


// constructs a list by copying the list v
template <class T> Light_List<T>::Light_List (Light_List<T> & l) {
	first = 0;
	last = 0;
	
	Node * q;
	q = l.first;
	
	Node * p;
	while (q != 0) {
		p = new Node;
		p->value = q->value; 
		p->next = 0;
		if (first == 0) first = p;
		else last->next = p;
		last = p;
		q = q->next;
	}
}

// constructs a List by copying the list v
template <class T> Light_List<T>::Light_List (const Light_List<T> & l) {
	first = 0;
	last = 0;
	
	Node * q;
	q = l.first;
	
	Node * p;
	while (q != 0) {
		p = new Node;
		p->value = q->value; 
		p->next = 0;
		if (first == 0)	first = p;
		else last->next = p;
		last = p;
		q = q->next;
	}
}


// destructor
template <class T> Light_List<T>::~Light_List () {
	Node * q = first;
	Node * p;
	
	while (q != 0)	{
		p = q;
		q = q->next;
		delete p;
	}
}


//-----------
// operators
//-----------

// operator =
template <class T> Light_List<T> & Light_List<T>::operator= (const Light_List<T> & l) {
	if (this != &l)	{
		// we remove the current list
		Node * q = first;
		Node * p;
		
		while (q != 0) {
			p = q;
			q = q->next;
			delete p;
		}
		first = 0;
		last = 0;
		
		// we build the new list by copying the list l
		q = l.first;
		
		while (q != 0) {
			p = new Node;
			p->value = q->value; 
			p->next = 0;
			if (first == 0)	first = p;
			else last->next = p;
			last = p;
			q = q->next;
		}
	}
	return *this;
}


//-----------
// iterators
//-----------


// returns an iterator referring to the first node in the list
template <class T> inline typename Light_List<T>::iterator Light_List<T>::begin () {
	return Light_List_Iterator<T> (this);
}

// returns an iterator referring to the position after the last node of the list
template <class T>
inline typename Light_List<T>::iterator Light_List<T>::end () {
	return Light_List_Iterator<T> (this,false);
}


//-----------------
// basic functions
//-----------------


// returns true if the list is empty, false otherwise
template <class T> inline bool Light_List<T>::empty () {
	return first == 0;
}


// adds the value t at the begin of the list
template <class T> inline void Light_List<T>::push_front (T t) {
	Node * p = new Node;
	p->value = t; 
	p->next = first;
	first = p;
	if (last == 0) last = p;
}


// adds the element referred by the iterator iter at the begin of the list
template <class T> inline void Light_List<T>::push_front (Light_List<T>::iterator iter)  {
	iter.current_pos->next = first;
	first = iter.current_pos;
	if (last == 0) last = iter.current_pos;
}


// adds the value t at the end of the list
template <class T> inline void Light_List<T>::push_back (T t) {
	Node * p = new Node;
	p->value = t; 
	p->next = 0;
	if (first == 0)	first = p;
	else last->next = p;
	last = p;
}


// adds the element referred by the iterator iter at the end of the list
template <class T> inline void Light_List<T>::push_back (Light_List<T>::iterator iter) {
	iter.current_pos->next = 0;
	if (first == 0)	first = iter.current_pos;
	else last->next = iter.current_pos;
	last = iter.current_pos;
}


// removes the element referred by the iterator iter
template <class T> void Light_List<T>::erase (Light_List<T>::iterator & iter) {
	if ((iter.object == this) && (iter.current_pos != 0)) {
		Node * p = iter.current_pos;
		
		if (iter.prec_pos == 0) {
			first = first->next;
			if (first == 0)	last = 0;
			iter.current_pos = first;
		}
		else {
				iter.prec_pos->next = p->next;
				iter.current_pos = p->next;
				
				if (last == p) last = iter.prec_pos;
			}
    delete p;
	}
}


// removes the element referred by the iterator iter from the list but does not destroy the object
template <class T> void Light_List<T>::skip (Light_List<T>::iterator & iter) {
	if ((iter.object == this) && (iter.current_pos != 0))	{
		Node * p = iter.current_pos;
		
		if (iter.prec_pos == 0)	{
			first = first->next;
			if (first == 0)	last = 0;
			iter.current_pos = first;
		}
		else {
				iter.prec_pos->next = p->next;
				iter.current_pos = p->next;
				
				if (last == p) last = iter.prec_pos;
			}
	}
}


// removes the element t from the list
template <class T> void Light_List<T>::remove (T t) {
	if (first != 0)	{
		if (first->value == t) {
			// the value t corresponds to the first element of the list
			Node * p = first;
			first = first->next;
			if (first == 0)  last = 0;			// case where the removed node is the last and unique node of the list
			
			delete p; 
		}
		else {
				Node * p = first->next;
				Node * prev = first;
				while ((p != 0) && (p->value != t))	{
					prev = p;
					p = p->next;
				}
				if (p != 0)	{
					prev->next = p->next;
					if (p == last)  last = prev;		// case where the removed node is the last node of the list
						
					delete p;
				}
			}
	}
}


// removes all the elements of the list
template <class T> void Light_List<T>::clear () {
	Node * q = first;
	Node * p;
	
	while (q != 0)	{
		p = q;
		q = q->next;
		delete p;
	}
	
	first = 0;
	last = 0;
}
