/// \file 
/// \brief Definitions related to the Light_Set class

#ifndef LIGHT_SET_H
#define LIGHT_SET_H

#include <light_vector.h>

template <class T>
class Light_Set: public Light_Vector<T>	{	/// This class allows to represent sets of elements (each element being of type T)
	protected:
		unsigned int card;		///< the cardinality of the set
		
	public:
		// constructors
		Light_Set ();					///< constructs an empty set
		Light_Set (int n);		///< constructs an empty set with at most n elements
		
		
		// operator
		Light_Set<T> & operator= (const Light_Set<T> & s);	///< operator =
		
		// iterators
		typedef Light_Iterator<T> iterator;		///< type for iterators on set
		Light_Set<T>::iterator begin();		///< returns an iterator referring to the first element of the set
		Light_Set<T>::iterator end();			///< returns an iterator referring to the position after the last element of the set
		
				
		// basic functions
		unsigned int size ();												///< returns the cardinality of the set
		void add (T val);														///< adds the value val to the set
		void remove (T val);												///< removes the value val from the set
		void remove (Light_Set<T>::iterator iter);	///< removes the value val selected by the iterator iter
		bool is_empty ();														///< returns true if the set is empty, false otherwise
		void clear ();															///< removes all the elements of the set
};

#include "light_set.tpp"

#endif

