//-----------------------------
// constructors and destructor
//-----------------------------

// constructs an empty vector
template <class T> Light_Vector<T>::Light_Vector () {
	capacity = 0;
	data = 0;
}


// constructs a vector of capacity n
template <class T> Light_Vector<T>::Light_Vector (int n) {
	capacity = n;
	data = new T [n];
}
	

// constructs a vector of capacity n and initializes each element to default_value
template <class T> Light_Vector<T>::Light_Vector (int n, T default_value) {
	capacity = n;
	data = new T [n];
	for (int i = 0; i < n; i++)		data[i] = default_value;
}


// constructs a vector by copying the vector v
template <class T> Light_Vector<T>::Light_Vector (Light_Vector<T> & v) {
	capacity = v.capacity;
	data = new T [capacity];
	
	for (unsigned int i = 0; i < capacity; i++)  data[i] = v.data[i];
}


// constructs a vector by copying the vector v
template <class T> Light_Vector<T>::Light_Vector (const Light_Vector<T> & v) {
	capacity = v.capacity;
	data = new T [capacity];
	
	for (unsigned int i = 0; i < capacity; i++) data[i] = v.data[i];
}


// destructor
template <class T> Light_Vector<T>::~Light_Vector () {
	delete [] data;
}


//-----------
// iterators
//-----------


// returns an iterator referring to the first element of the set
template <class T> inline Light_Iterator<T> Light_Vector<T>::begin() {
	return Light_Iterator<T> (this);
}


// returns an iterator referring to the position after the last element of the set
template <class T> inline Light_Iterator<T> Light_Vector<T>::end() {
	return Light_Iterator<T> (this,capacity);
}


//-----------
// operators
//-----------


// operator =
template <class T> Light_Vector<T> & Light_Vector<T>::operator= (const Light_Vector<T> & v) {
	if (this != &v)	{
		if (capacity != v.capacity)	{
			delete [] data;
			capacity = v.capacity;
			data = new T [capacity];
		}
		
		for (unsigned int i = 0; i < capacity; i++)  data[i] = v.data[i];
	}
	return *this;
}


// operator []
template <class T> inline T& Light_Vector<T>::operator[](int n) {
	return data[n];
}


// operator []
template <class T> inline const T& Light_Vector<T>::operator[] (int n) const {
	return data[n];
}


//-----------------
// basic functions
//-----------------


// returns the capacity of the vector
template <class T> inline unsigned int Light_Vector<T>::get_capacity () {
	return capacity;
}


// expands the vector by adding n elements at the end of the vector
template <class T> void Light_Vector<T>::expand (int n) {
	if (n > 0) {
		T * old_data = data;
		data = new T [capacity+n];
		for (unsigned int i = 0; i < capacity; i++)  data[i] = old_data[i];
		
		delete [] old_data;
		
		capacity += n;
	}
}


// expands the vector by adding n elements at the end of the vector and initializes each new element to default_value
template <class T> void Light_Vector<T>::expand (int n, T default_value) {
	if (n > 0) {
		T * old_data = data;
		
		data = new T [capacity+n];
		for (unsigned int i = 0; i < capacity; i++)  data[i] = old_data[i];

		for (int i = 0; i < n; i++)  data[i+capacity] = default_value;
		
		delete [] old_data;
		
		capacity += n;
	}
}


// swaps the values at indices i and j
template <class T> void Light_Vector<T>::swap (int i, int j) {
	T tmp = data[i];
	data[i] = data[j];
	data[j] = tmp;
}


// adds the value val at this end of the vector
template <class T> void Light_Vector<T>::push_back (T val) {
	expand (1);
	data[capacity-1] = val;
}


// removes the value val from the vector
template <class T> void Light_Vector<T>::erase (T val) {
	unsigned int i = 0;
	
	while ((i < capacity) && (data[i] != val))  i++;
	
	if (i < capacity)	{
		capacity--;
		while (i < capacity) {
			data[i] = data[i+1];
			i++;
		}
	}
}
